package net.java.dev.fwutil.demoapp; 

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.lang.reflect.Method;
import java.net.URL;
import java.net.URLClassLoader;

/**
 * Loads classes from jar-files and jar-files contained within another jar-file.
 * Copy this class to your application's package that has the class with the main-method.
 * Use the methods in the copy of this class to load required classes, e.g.
 * by calling addJarResource("/fwutil.jar").
 * 
 * Copied and adjusted from http://stackoverflow.com/questions/60764/how-should-i-load-jars-dynamically-at-runtime
 * @author Fred
 *
 */
public class SysJarLoader {

	public static String TempDirName = "";

	@SuppressWarnings("unchecked")
	private static final Class[] parameters = new Class[] {URL.class};

	public static final URLClassLoader sysloader = (URLClassLoader) URLClassLoader.getSystemClassLoader();

	/** Searches a directory for jar-files and adds them to the system class loader. 
	 * @return The number of jar-files added to the class loader.
	 */
	public static int addLibDir(String libDir) throws Exception {

		File dependencyDirectory = new File(libDir);
		File[] files = dependencyDirectory.listFiles();
		if (files == null) throw new IOException("Lib-directory does not exist: " + dependencyDirectory.getCanonicalPath());
		int numberOfFiles = 0;
		for (int i = 0; i < files.length; i++) {
		    if (files[i].getName().toLowerCase().endsWith(".jar")) {
		        //urls.add(files[i].toURL());
		        addFile(files[i].getName());
		        numberOfFiles++;
		    }
		}
		return numberOfFiles;
	}
	
	/** Adds the classes in the jar-file specified by resourceName
	 * to the class loader. The specified resource is copied to a file
	 * in the java-temp directory in order to load the classes. 
	 * <br>The resourceName should start with a / when the resource is 
	 * located at the root of the runtime jar. 
	 * @param resourceName
	 * @throws Exception
	 */
	public static void addJarResource(String resourceName) throws Exception {
		
		URL jarResource = sysloader.getResource(resourceName);
		if (jarResource == null)
			jarResource = Thread.currentThread().getContextClassLoader().getResource(resourceName);
		if (jarResource == null)
			jarResource = SysJarLoader.class.getResource(resourceName);
		if (jarResource == null) {
			throw new IOException("Could not find resource " + resourceName);
		}
		URL fileResource = new File(copyResourceToTempFile(jarResource)).toURI().toURL();
		addURL(fileResource); 
		System.out.println("Added " + resourceName + " to class-loader.");
	}

	public static void addFile(String fileName) throws IOException {
		File f = new File(fileName);
		addFile(f);
	}

	public static void addFile(File f) throws IOException {
		addURL(f.toURI().toURL());
	}

	@SuppressWarnings("unchecked")
	public static void addURL(URL u) throws IOException {
		
		Class sysclass = URLClassLoader.class;
		try {
			Method method = sysclass.getDeclaredMethod("addURL", parameters);
			method.setAccessible(true);
			method.invoke(sysloader, new Object[] {u});
		} catch (Throwable t) {
			//t.printStackTrace();
			throw new IOException("Could not add URL [" + u + "] to system classloader", t);
		}
	}
	
	/** Utility function used to copy a resource jar file to a temp-file. */
	public static String copyResourceToTempFile(URL u) throws IOException {
		
		InputStream in = null;
		OutputStream out = null;
		String outName = null;
		try {
			in = u.openStream();
			outName = new File(u.getFile()).getName();
			outName = System.getProperty("java.io.tmpdir") + File.separator + 
				TempDirName + File.separator + outName;
			out = new FileOutputStream(outName);
			copyStreams(in, out);
		} finally {
			if (in != null) try { in.close(); } catch (Exception ignored) {}
			if (out != null) try { out.close(); } catch (Exception ignored) {}
		}
		return outName;
	}
	
	/** Utility function that copies bytes from in to out. */
	public static void copyStreams(InputStream in, OutputStream out) throws IOException {
		
		byte[] buf = new byte[65536];
		int len;
		while((len = in.read(buf)) > 0) {
			out.write(buf, 0, len);
		}
	}
}
