/*
 * This software is licensed under the GNU Lesser GPL, see
 * http://www.gnu.org/licenses/ for details.
 * You can use this free software in commercial and non-commercial products,
 * as long as you give credit to this software when you use it.
 * This softwware is provided "as is" and the use of this software is 
 * completely at your own risk, there are absolutely no warranties.
 * 
 */

package net.java.dev.fwutil;

import java.awt.Container;
import java.awt.Dialog;
import java.awt.Font;
import java.awt.Frame;
import java.awt.GraphicsEnvironment;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.SwingUtilities;
//import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.ListSelectionModel;

import net.java.dev.hvlayout.*;

/**
 * A modal dialog for selecting a font size and type. The selected font size and
 * type are stored in Settings (via Settings.defaultFontSize and
 * Settings.defaultFontName) and the UI is updated.
 * When a different font size is selected, a UI update is usually only
 * partially succesfull. A complete restart (exit and launch again) is required
 * to re-draw the existing windows.
 * <p> 
 * Settings.setDefaultFont() is called during startup so that the font
 * size and type selected in this dialog is applied during startup. Note
 * that if only a different font size is selected, all font types are
 * unchanged (i.e. whatever UIManager has loaded as font type remains, just
 * the size is updated). 
 * <p>
 * Note that many people require a large font to easily read the text shown
 * on screen. In fact, not being able to show text using a large font
 * could seriously impact the usability of your program. Many people have
 * less then perfect eyesight and a big font can prevent many sore eyes and
 * headaches. 
 * <p>
 * FontSelector uses the hvlayout package and the OkCancelDialog.
 * <br> The FontSelector dialog can be resized, see the method 
 * buildSelectorPane() on how this is achieved using the hvlayout package.
 * <br> This class can be executed stand-alone to get an impression of how this
 * font selection dialog looks (see bin\demofontselector.bat).
 * <br> The example text is loaded via Labels since UTF-8 is required to 
 * load the various international text examples. I found that on Windows, 
 * only Dialog, Monospaced and (Sans)Serif font type could display all languages.
 * <p>
 * Part of the source code was copied from 
 * <br>http://www.java2s.com/Code/Java/Tiny-Application/Afontselectiondialog.htm
  * @author Fred
 *
 */
public class FontSelector implements ActionListener,
									ListSelectionListener{

	private static final long serialVersionUID = 1L;
	private static boolean initialized = false;
	private static JList fontSizeList;
	private static JList fontNameList;
	private static boolean newFontSize;
	private static OkCancelDialog dialog;
	private static JButton ok;
	private static JButton cancel;
	private static HVTextArea exampleText;
	private static JScrollPane exampleTextScroller;
	private static JCheckBox reset;
	/** The list of Fonts */
	protected static String fontNames[];
	/** The list of font sizes */
	protected static final String fontSizes[] = { "8", "10", "11", "12", "14", "16", "18",
	      "20", "24", "30", "36", "40", "48", "60", "72" };
	protected static final int fontiSizes[] = { 8, 10, 11, 12, 14, 16, 18,
	      20, 24, 30, 36, 40, 48, 60, 72 };

	/** The font-demonstration text (label FontShowText) */
	protected static String displayText;
	private static int selectedFontSize;
	private static String selectedFontName;
	private static VBox mainPane;
	/** This is a singleton class */
	private static FontSelector thisInstance = new FontSelector();
	
	private FontSelector() {

		super();
	}
	
	/**
	 * Loads text and configures components.
	 *
	 */
	private static void initialize() {
		
		if (initialized)
			return;
		displayText = Labels.get("FontShowText");
		reset = FormUI.getCheckBox(thisInstance, Labels.get("ResetFont"), 0);
		fontSizeList = new JList(fontSizes);
		fontSizeList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		fontSizeList.addListSelectionListener(thisInstance);
		// For JDK 1.1: returns about 10 names (Serif, SansSerif, etc.)
	    // fontList = toolkit.getFontList();
	    // For JDK 1.2: a much longer list; most of the names that come
	    // with your OS (e.g., Arial), plus the Sun/Java ones (Lucida,
	    // Lucida Bright, Lucida Sans...)
	    fontNames = GraphicsEnvironment.getLocalGraphicsEnvironment()
	        .getAvailableFontFamilyNames();
	    fontNameList = new JList(fontNames);
	    fontNameList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	    fontNameList.addListSelectionListener(thisInstance);
		
		exampleText = FormUI.getTextArea(displayText, false);
		HVProps.doNotShrinkContainer(exampleText);
		exampleTextScroller = HVProps.getScroller(exampleText, exampleText.getPreferredSize().width, 8, false);
		initialized = true;
	}

	/**
	 * Builds the main pane using the components configured in initialize().
	 */
	private static Container buildSelectorPane() {
	
		mainPane = new VBox();
		mainPane.add(FormUI.getLabel(Labels.get("FontMessage"), 0));
		reset.setSelected(false);
		mainPane.add(reset);
		
		HBox lb = new HBox();
		VBox lbl = new VBox();
		lbl.add(FormUI.getLabel(Labels.get("FontSize"), 50));
		lbl.add(HVProps.getScroller(fontSizeList, 50, 10, false));
		lb.add(lbl);
		lbl = new VBox();
		lbl.add(FormUI.getLabel(Labels.get("FontName"), 0));
		lbl.add(HVProps.getScroller(fontNameList, 0, 10, false));
		lb.add(lbl);
		mainPane.add(lb);
		mainPane.add(dialog.getOkCancelLine());
		mainPane.add(FormUI.getTextArea(Labels.get("FontDisplayText"), false));
		mainPane.add(exampleTextScroller);
		return mainPane;
	}
	
	/**
	 * Shows the font selector dialog.
	 * At the moment, there is no way of knowing if the user selected a different
	 * font name from the return value. 
	 * @return The default font size selected by the user, 0 if no change was made.
	 */
	public static int show(Frame owner) {
		
		initialize();
		newFontSize = false;
		dialog = new OkCancelDialog(owner, Labels.get(Settings.defaultFontSize), thisInstance);
		ok = dialog.getOkButton();
		cancel = dialog.getCancelButton();
		
		// Add the fotn selector components.
		dialog.setContentPane(buildSelectorPane());
		
		fontSizeList.requestFocusInWindow();
		
		selectedFontSize = HVProps.getDefaultFontSize();
		selectedFontName = HVProps.getDefaultUIFont().getFamily();
		
		// Find the current font (size) and select them in the list.
		int selectedPos = 0;
		while ((selectedPos < fontSizes.length
				&& Integer.parseInt(fontSizes[selectedPos]) != selectedFontSize))
			selectedPos++;
		if (selectedPos < fontSizes.length) {
			fontSizeList.setSelectedIndex(selectedPos);
			fontSizeList.ensureIndexIsVisible(fontSizeList.getSelectedIndex());
		} else
			Log.logWarning("FontSelector could not find current default font size in graphics environment.");
	    selectedPos = 0;
	    
	    while ((selectedPos < fontNames.length)
	    		&& !fontNames[selectedPos].equals(selectedFontName))
	    		selectedPos++;
		if (selectedPos < fontNames.length) { 
			fontNameList.setSelectedIndex(selectedPos);
			fontNameList.ensureIndexIsVisible(fontNameList.getSelectedIndex());
		} else
			Log.logWarning("FontSelector could not find current default font family name in graphics environment.");
		
		dialog.pack();
		dialog.setVisible(true);
		dialog.dispose();
		dialog = null;
		if (newFontSize)
			return selectedFontSize;
		return 0;
	}
	
	/**
	 * Updates the example text with selected font and size.
	 *
	 */
	private void updateExampleText() {

		exampleTextScroller.remove(exampleText);
		
		Font newFont = new Font(selectedFontName, Font.PLAIN, selectedFontSize);
		int currentFSize = HVProps.getDefaultFontSize();

		HVProps.setDefaultFontSize(selectedFontSize);
		exampleText = FormUI.getTextArea(displayText, newFont);
		HVProps.setDefaultFontSize(currentFSize);
		exampleTextScroller.setViewportView(exampleText);
	}

	/**
	 * Listens for OK and Cancel events.
	 * If OK is selected:
	 *  - Settings.defaultFontSize and Settings.defaultFontSize is set
	 *  - The UI is updated.
	 *  If reset was selected, font size Settings are cleared.
	 */
	public void actionPerformed(ActionEvent action) {
		
		Object as = action.getSource();

		if (as == cancel) {
			dialog.setVisible(false);
		} else if (as == ok) {
			if (reset.isSelected()) {
				Settings.remove(Settings.defaultFontSize);
				Settings.remove(Settings.defaultFontName);
			} else {
				if (HVProps.getDefaultFontSize() != selectedFontSize) { 
					HVProps.setDefaultUIFontSize(selectedFontSize);
					Settings.put(Settings.defaultFontSize, new Integer(selectedFontSize));
					newFontSize = true;
				} else
					newFontSize = false;
				if (!HVProps.getDefaultUIFont().getFamily().equals(selectedFontName)) {
					Font newFont = new Font(selectedFontName, Font.PLAIN, selectedFontSize);
					HVProps.setDefaultUIFont(newFont);
					Settings.put(Settings.defaultFontName, selectedFontName);
					updateUI();
				} else
					if (newFontSize)
						updateUI();
			}
			dialog.setVisible(false);
		} else if (as == reset) {
			fontSizeList.setEnabled(!reset.isSelected());
			fontNameList.setEnabled(!reset.isSelected());
		}

	}

	/**
	 * Listens for selected font (size) and updates
	 * the example text to use the selected font (size).
	 */
	public void valueChanged(ListSelectionEvent event) {
		
		if (event.getSource() == fontSizeList) {
			int selectedPos = fontSizeList.getSelectedIndex();
			if (selectedPos > -1) {
				selectedFontSize = Integer.parseInt(fontSizes[selectedPos]);
				updateExampleText();
			}
		} else if (event.getSource() == fontNameList) {
			int selectedPos = fontNameList.getSelectedIndex();
			if (selectedPos > -1) {
				selectedFontName = fontNames[selectedPos];
				updateExampleText();
			}
		}
	}
	
	/**
	 * Attempts to update the UI with the selected font (size) by
	 * calling SwingUtilities.updateComponentTreeUI(Window) for all Windows.
	 * This usually only works partially, a complete restart is needed.
	 */
	public static void updateUI() {
		Window windows[] = Dialog.getWindows();
		for (int i = 0; i < windows.length; i++) {
			SwingUtilities.updateComponentTreeUI(windows[i]);
		}
	}
	
	public static void main(String args[]) {
		net.java.dev.fwutil.Startup.boot(args, "FontSelector", "0.1", "");
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				show(null);
			}
		});
	}
}
