/*
 * This software is licensed under the GNU Lesser GPL, see
 * http://www.gnu.org/licenses/ for details.
 * You can use this free software in commercial and non-commercial products,
 * as long as you give credit to this software when you use it.
 * This softwware is provided "as is" and the use of this software is 
 * completely at your own risk, there are absolutely no warranties.
 * 
 */

package net.java.dev.fwutil;

import java.io.InputStream;
import java.util.Enumeration;
import java.util.Hashtable;

import net.java.dev.fwutil.Labels;

/**
 * Singleton class that provides convenience methods for Labels. 
 * 
 * Consistent use of this class will allow for localization 
 * (simply copy the original label-file and start translating).
 * The default label-file conf\labels.txt (loaded by StartUp.boot()) 
 * provides examples of how labels can be specified.
 * <br> Run this class to check a label-file. See the checkLabels()
 * method for details.
 * <br> When a label is requested but not found, missingLabel is returned. 
 * 
 */
public class Labels {
	
	@SuppressWarnings("unused")
	private static Labels thisInstance = new Labels();
	
	/** Default name of label-file (located in the conf-directory). */
	public static final String defaultLabelFile = "labels.txt";
	
	/** Default value when a label could not be found. */
	public static String missingLabel = "Missing label: ";
	
	/** Default filename to read labels from. Used in combination with
	    Util.confDir to locate default label-file. */
	public static String labelFile = defaultLabelFile;
	
	/** Used when check-labels is used, prevents loading the default labels. */
	public static boolean skipLoad = false;
	
	/** Contains all labels. */
	public static Hashtable<String, String> labels = new Hashtable<String, String>();
	
	/** This is a singleton class. */
	private Labels() {
		
		super();
	}

	/**
	 * Uses default label file in conf-dir to load labels from. Unless
	 * a file-name other then the default has been specified in which case
	 * this file-name is used directly (the Util.confDir is NOT prepended).  
	 * See initialized(fileName)
	 * @return true if labels have been loaded from file
	 */
	public static boolean initialize() {

		InputStream lf = null;
		
		if (labelFile.equals(defaultLabelFile))
			lf = Util.getFileStream(Util.confDir + "/" + labelFile);
		if (lf == null)
			lf = Util.getFileStream(labelFile);
		return initialize(lf);
	}
	
	/**
	 * Loads labels from an UTF-8 encoded file. Does not load labels when skipLoad is true (required
	 * when using checkLabels(...), see also main-method.
	 * See Util.bigStringToParameters for details (this method converts the file
	 * to key-value pairs).
	 * @param fileName the file to read labels from
	 * @return true if labels have been loaded from file
	 */
	public static boolean initialize(InputStream in) {
		
		Hashtable<String, String> loadLabels;
		if (skipLoad) 
			return true;
		loadLabels = Util.bigStringToParameters(Util.readFileToString(in, Util.utf8Encoding), false);
		if (loadLabels == null)
			return false;
		labels = loadLabels;
		return true;
	}
	
	/** Removes all labels from memory. */
	public static void clear() {
		labels.clear();
	}
	
	/**
	 * Gets a label from memory.
	 * @param key The label-key
	 * @return The label-text or missingLabel + key when the label could not be found.
	 */
	public static String get(String key) {
		
		String s = labels.get(key);
		if (s == null)
			return missingLabel + key;
		return s;
	}
	
	/**
	 * Adds a label-key and value to memory.
	 * Note that there is not method available for storing added labels. This
	 * has to be done by editing the label-file.
	 * @param key may not be null or an empty string
	 * @param label may not be null
	 */
	public static void add(String key, String label) {
		
		if (key != null && !key.equals(Util.emptyString) && label != null)
			labels.put(key, label);
	}
	
	/**
	 * Checks a label-file (called by main method, see also bin\checklabels.bat) 
	 * <br> Checks for:
	 * <br>   -keys used more then once
	 * <br>   -keys without a value
	 * <br> Provides extra output for multi-line labels.
	 * <br> Displays labels with "i18n" in their name (useful for checking
	 * proper UTF-8 encoding).
	 * @param fileName the file containing the labels.
	 */
	@SuppressWarnings("unchecked")
	public static void checkLabels(String fileName) {
		
		Hashtable<String, String> loadLabels = new Hashtable<String, String>();
		Log.log("Label checker requires the name of the file containg the labels as runtime argument, e.g. \n -labels=C:\\eclipse\\ws\\conf\\labels.txt");
		String bigString = null;
		bigString = Util.readFileToString(fileName);
		if (bigString == null) {
			Log.logError("Could not open " + fileName);
			return;
		}
		Log.log("Checking label-file: " + fileName);
		Settings.appDebug = true;
		/* Instead of using Util.bigStringToParameters
		 * now go through the label-file line by line so that
		 * the errors can be reported with line-number 
		 * and so that extra checks can be done for multi-line labels.
		 */
		StringLine.FileStringIterator fsi = StringLine.getNewFSI(bigString, 0);
		int bigStringLength = fsi.bigString.length();
		String[] keyValue = new String[2];
		String line = StringLine.getNextLine(fsi);
		while (line != null) {
			try {
				if (Util.getKeyValue(line, keyValue, false)) {
					if (loadLabels.get(keyValue[0]) != null)
						Log.logError("At line " + fsi.lineCount +":" + Util.space + "Key used more then one time: " + keyValue[0]);
					if (keyValue[1].equals(Util.emptyString))
						Log.logWarning("At line " + fsi.lineCount +":" + Util.space + "No value for key: " + keyValue[0]);
					if (keyValue[1].endsWith(Util.continueTextValue)) {
						if (Settings.appDebug)
							Log.log("Continue reading multi line value for " + keyValue[0] + " ...");
						keyValue[1] = Util.continueReadMultiLineValue(keyValue[1], fsi, bigStringLength);
//						if (Settings.appDebug)
//							Log.log("Multi-line value read: " + keyValue[1]);
					}
					loadLabels.put(keyValue[0], keyValue[1]);
				}
			} catch (Exception e) {
				String s = "At line " + fsi.lineCount +":" + Util.space + e;
				Log.logWarning(s);
			}
			line = StringLine.getNextLine(fsi);
		}
		Log.log("Lines read:" + fsi.lineCount);
		Log.log("Checked keys: " + Integer.toString(loadLabels.size()));
		Log.log("Done checking");
		Log.log("International labels loaded: ");
		Enumeration keyList = loadLabels.keys();
		while (keyList.hasMoreElements()) {
			String key = (String) keyList.nextElement();
			if (key.startsWith("I18n")) {
				String value = loadLabels.get(key);
				Log.log(key + ": " + value);
			}
		}
	}
	
	/**
	 * Checks the labelfile specified by the labels argument. E.g.:
	 * <br> bin\checklabels.bat -labels=C:\eclipse\ws\conf\labels.txt
	 * <br> If no argument is specified, the default label-file is loaded
	 * (located in .\conf\labels.txt). The label-file delivered with this
	 * package contains a couple of on-purpose-errors that are used to test the
	 * checklabels method.
	 * <p>
	 * Note that the log first shows a couple of error-messages.
	 * This is because the normal load-labels procedure is first executed 
	 * without loading labels and then the the check-labels procedure
	 * is called (which loads the labels from the specified file). 
	 * Start reading the messages that appear below:
	 * <br> Checking label-file: filename
	 */
	public static void main(String[] args) {
		
		skipLoad = true;
		Startup.boot(args, "Label file checker", "0.1", "");
		checkLabels(Labels.labelFile);
	}
}
