/*
 * This software is licensed under the GNU Lesser GPL, see
 * http://www.gnu.org/licenses/ for details.
 * You can use this free software in commercial and non-commercial products,
 * as long as you give credit to this software when you use it.
 * This softwware is provided "as is" and the use of this software is 
 * completely at your own risk, there are absolutely no warranties.
 * 
 */

package net.java.dev.fwutil;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;


/**
 * A log-file (singleton class). Uses LogUI to show log messages.
 * Log-files are rotated and stored in the application data directory.
 * Output to System.out and System.err is also captured by this log
 * (this is done using the LogJavaSystem class).
 * If Settings.appDebug is true, the output to System.out and System.err
 * remains and this log makes a copy.
 * Change the static variables in this class to suit your log-file needs.
 * <br>
 * Any method call to this class will open the LogUI window.
 * <p>
 * If rotating the log-files fails, this is usually because an application
 * did not close properly and still has a lock on one of the log-files.
 * Killing the hanging process solves this.
 * <p>
 * Note that each log-message is written and committed to file (else
 * how do you know what caused the application to stop/crash?),
 * Writing and committing to file is a blocking process (java.nio is not used). 
 * Logging to much messages could seriously impact application 
 * performance (e.g. when logging a lot of debug-messages).
 * 
 * @author Fred
 *
 */
public final class Log {
	
	/** The names of the rotating log-files. Change/Add and/or remove a name to
	 * use more/less log-files. The first log-file is always the most recent.
	 * There must be at least 2 log-files.
	 */ 
	public static String logFileName[] = {"log0.txt", "log1.txt", "log2.txt", "log3.txt"};
	/** The maximum size of 1 log-file. The default value (256k) is best for performance.
	 * Any bigger is likely to cause delays. 
	 */
	public static int logFileMaximumSize = 262144;
	
	private static final long serialVersionUID = 1L;

	private static Log thisInstance = new Log();

	/** The number of stack-traces to log from an exception. */
	public static final int maxStackTrace = 8; 
	private static String logDir = null;
    private static BufferedWriter logWriter = null;
    private static boolean logFileRename = true;
    private static File logFile;
    private static LogJavaSystem systemErrStream;
    private static LogJavaSystem systemOutStream;
    
	private Log() {
	
		super();
		LogUI.getInstance(); // Causes log-window to show.
	}
	
	/** This is a singleton class with one instance. */
	public static Log getInstance() {
		
		return thisInstance;
	}
	
	/** Opens the log-files and starts capturing output to System.out/err.
	 * 
	 * @param logDir The directory to store log-files.
	 * @param logMessages The first message to store in the log-files (e.g. "application starting up").
	 * @return True if log-files were opened.
	 */
	public static boolean initialize(String logDir, String logMessages) {
		
		if (logWriter == null) {
			Log.logDir = logDir;
			if (openLog()) {
				log(logMessages);
				if (Settings.appDebug) log("Application debug enabled");
				log("Log file opened: " + getLogFileName());

				// if appDebug, then LogJavaSystem should log to screen AND the log-file.
				systemErrStream = new LogJavaSystem(true, Settings.appDebug);
				systemOutStream = new LogJavaSystem(false, Settings.appDebug);
				return true;
			} else {
				return false;
			}
		}
		return true;
	}
	
	/** Returns the full path to the most recent log-file. */
	public static String getLogFileName() {
		
		return logDir + Util.fileSeparator + logFileName[0];
	}
	
	/** Open the log-file. */
	private static boolean openLog() {
	
		boolean error = false;
		logFile = new File(getLogFileName());
		try {
			if (logFile.exists()) {
				logWriter = new BufferedWriter(new FileWriter(getLogFileName(), true));
			} else {
				logWriter = new BufferedWriter(new FileWriter(getLogFileName()));
			}
		} catch (IOException ieo) {
			try {
				if (logWriter != null) logWriter.close();
			} catch (IOException unused) {
				error = true;
			}
			logWriter = null;
			log("Failed to open log file: " + getLogFileName());
			logError(ieo);
			error = true;
		}
		return (!error);
	}
	
	/** Hides the log-window. */
	public static boolean makeInvisible() {

		if (LogUI.makeInvisible()) {
			if (Settings.appDebug)
				log("Log window hide");
			return true;
		}
		return false;
	}
	
	/** Shows the log-window. */
	public static boolean makeVisible() {

		if (LogUI.makeVisible()) {
			if (Settings.appDebug)
				log("Log window show");
			return true;
		}
		return false;
	}
	
	/**
	 * Closes the log-files and stops listening to System.out/err
	 * Closes the log-window.
	 *
	 */
	public static void close() {
	
		if (logWriter != null) {
			log("Log closing");
			systemErrStream.close();
			systemOutStream.close();
			try {
				logWriter.flush();
				logWriter.close();
			} catch (IOException unused) {}
			logWriter = null;
		}
		LogUI.close();
	}
	
	/** Converts maxStackTrace amount of stack trace elements to a String */ 
	private static String stackTrace2String(StackTraceElement[] ste) {
		
		StringBuffer s = new StringBuffer("");
		if (ste.length > 0) {
			s.append(Util.newline + " at " + ste[0].toString());
			int i = 1;
			while (i < ste.length && i < maxStackTrace) {
				s.append(Util.newline + "  " + ste[i].toString());
				i++;
			}
			if (i < ste.length)
				s.append(Util.newline + " with " + Integer.toString(ste.length - i) + " more additional trace elements.");
		}
		return s.toString();
	}
	
	/**
	 * Converts an exception to a String. A maximum of maxStackTrace lines from
	 * the exception are copied into the String (if all stack trace elements are logged,
	 * a log quickly becomes unreadable).
	 * @param e The exception/throwable.
	 * @return A String reflecting the exception.
	 */
	public static String exception2String(final Throwable e) {
		
		// Message can be null (with NullPointerException) which lets StringBuffer fail to initialize. 
		StringBuffer s = new StringBuffer(getExceptionDescription(e));
		StackTraceElement[] ste =  e.getStackTrace();
		s.append(stackTrace2String(ste));
		Throwable cause = e.getCause();
		while (cause != null) {
			s.append(Util.newline + " caused by: " + getExceptionDescription(cause));
			if (cause.getCause() == null) {
				ste = cause.getStackTrace();
				s.append(stackTrace2String(ste));
			}
			cause = cause.getCause();
		}
		return s.toString();
	}
	
	/** Returns the exception class-name with the error-message. 
	 * This is the same as e.toString() but this method is sometimes overridden
	 * in which case crucial information can get lost. */
	public static String getExceptionDescription(final Throwable e) {
		
		String s = e.getClass().getName();
		if (e.getMessage() != null)
			s = s + ": " + e.getMessage();
		return s;
	}
	
	/**
	 * Writes the given exception as error in the log.
	 * Converts the exception to a String using exception2String()
	 */
	public static void logError(Throwable e) {
		
		logError(exception2String(e));
	}
	
	/**
	 * Writes the given String as error in the log.
	 */
	public static void logError(String message) {
		/*
		 * This method is called directly from LogJavaSystem.
		 * An option should be present to notify the user of System.err messages
		 * but this requires the message to be parsed
		 * (e.g. only show first line) and passed on to
		 * UIMessages.showErrorMessage
		 * But this in turn gives a possible circular dependency on UIMessages. 
		 */
		
		log("ERROR:" + Util.newline + message);
	}
	
	/**
	 * Writes the given exception as warning in the log.
	 * Converts the exception to a String using exception2String()
	 */
	public static void logWarning(Exception e) {
		
		logWarning(exception2String(e));
	}
	
	/**
	 * Writes the given String as warning in the log.
	 */
	public static void logWarning(String message) {
		
		log("Warning:" + Util.newline + message);
	}
	
	/** Updates the UI to show the log-message. */
	private static void showMessage(String message) {
		
		LogUI.showMessage(message);
	}
	
	/** Rotates the log-files. */
	private static boolean rotateLogFiles() {
		
		int lastLogFile = logFileName.length;
		boolean lastLogFileFound = false;
		while (lastLogFile > 0 && !lastLogFileFound) {
			lastLogFileFound = new File(logDir + Util.fileSeparator + logFileName[lastLogFile - 1]).exists();
			lastLogFile--;
		}
		if (lastLogFile == logFileName.length - 1) {
			if (!new File(logDir + Util.fileSeparator + logFileName[lastLogFile]).delete()) {
//				System.out.println("Delete failed " + logFileName[lastLogFile]);
				return false;
			}
			lastLogFile--;
		}
		boolean error = false;
		File from;
		File to;
		while (!error && lastLogFile >= 0) {
			from = new File(logDir + Util.fileSeparator + logFileName[lastLogFile]);
			to = new File(logDir + Util.fileSeparator + logFileName[lastLogFile + 1]);
			error = !from.renameTo(to);
			lastLogFile--;
		}
		return !error;
	}
	
	/**
	 * Writes the given message to file and shows it in the log-window.
	 * Rotates log-files when logFileMaximumSize is exceeded.
	 * When log-files are rotated, the text in the log-window is reduced to 10%
	 * of the last shown text (for performance reasons).  
	 */
	public static void log(String message) {
		
		if (message == null || message.equals(Util.emptyString)) 
			message = "Got empty message to log";
		String timeStamp;
		if (Settings.appDebug)
			timeStamp = Util.debugDateTime();
		else
			timeStamp = Util.shortDateTime();
		showMessage(timeStamp + Util.space + message);
		if (logWriter != null) {
			String s = Util.simpleDateTime() + Util.space + message + Util.newline;
			char[] c = s.toCharArray();
			try {
				logWriter.write(c, 0, c.length);
				logWriter.flush();
			} catch (IOException ioe) {
				showMessage("Failed to write to log-file: " + ioe.getMessage() + Util.newline);
			}
			boolean error = false;
			if (logFileRename && logFile.length() > logFileMaximumSize) {
				showMessage("Log file exceeded maximum size, creating new log file" + Util.newline);
				try {
					logWriter.close();
				} catch (IOException ioe) {
					showMessage("Error closing log-file: " + ioe.getMessage());
				}
				logWriter = null;
				if (!rotateLogFiles()) {
					showMessage("Log files could not be renamed, log file closed!");
					error = true;
				}
				if (!error) {
					try {
						logWriter = new BufferedWriter(new FileWriter(logFile.getAbsolutePath()));
					} catch (IOException ieo) {
						if (logWriter != null) {
							try {
								logWriter.close();
							} catch (IOException unused) {}
						}
						logWriter = null;
						error = true;
					}
				}
				if (error) {
					logFileRename = false;
					showMessage("Failed to open log-file " + logFile.getAbsolutePath() + Util.newline);
				} else {
					LogUI.chopText(logFileMaximumSize);
					log("Log files have rotated");
				}
			}
		}
	}
}