/*
 * This software is licensed under the GNU Lesser GPL, see
 * http://www.gnu.org/licenses/ for details.
 * You can use this free software in commercial and non-commercial products,
 * as long as you give credit to this software when you use it.
 * This softwware is provided "as is" and the use of this software is 
 * completely at your own risk, there are absolutely no warranties.
 * 
 */

package net.java.dev.fwutil;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;

import java.util.Timer;
import java.util.TimerTask;


/**
 * Listens (in debug-mode) or captures all output to System.out or System.err.
 * The output is stored in a buffer for 0.3 seconds (to collect the rest of the message)
 * and then flushed/forwarded to the log. A thread/TimerTask is used to accomplish this 
 * (classic example on how to use thread, Timer and TimerTask).
 * <br> System.err is forwarded to the log as error message.
 * <p>
 * This class has been copied and modified from 
 * <br> http://www.mail-archive.com/log4j-user@logging.apache.org/msg05005.html
 * @author Fred
 *
 */
public class LogJavaSystem extends OutputStream {

	/**
	 * Used to maintain the contract of [EMAIL PROTECTED] #close()}.
	 */
	protected boolean hasBeenClosed = false;

	/**
	 * The internal buffer where data is stored.
	 */
	protected byte[] buf;

	/**
	 * The number of valid bytes in the buffer. This value is always
	 * in the range <tt>0</tt> through <tt>buf.length</tt>; elements
	 * <tt>buf[0]</tt> through <tt>buf[count-1]</tt> contain valid
	 * byte data.
	 */
	protected int count;

	/**
	 * Remembers the size of the buffer for speed.
	 */
	private int bufLength;

	/**
	 * The default number of bytes in the buffer.
	 */
	public static final int DEFAULT_BUFFER_LENGTH = 2048;
	
	protected PrintStream org;
	private boolean systemErr;
	private boolean writeToOriginalStreamAlso;
	private boolean flushTaskWaiting;
	
	private static Timer logtimer;
	public static final long flushDelay = 300;

	@SuppressWarnings("unused")
	private LogJavaSystem() {
		// Should not be called
	}
	
	/**
	 * Makes this class buffer output to System.out/err
	 * @param systemErr If true, listen to System.err else System.out
	 * @param writeToOriginalStreamAlso If true, does not stop output to System.out/err.
	 */
	public LogJavaSystem(boolean systemErr, boolean writeToOriginalStreamAlso) {

		super();
		bufLength = DEFAULT_BUFFER_LENGTH;
		buf = new byte[DEFAULT_BUFFER_LENGTH];
		count = 0;
		this.systemErr = systemErr;
		this.writeToOriginalStreamAlso = writeToOriginalStreamAlso;
		if (systemErr) {
			org = System.err;
			System.setErr(new PrintStream(this));
		} else {
			org = System.out;
			System.setOut(new PrintStream(this));
		}
		logtimer = Util.getTimer();
		flushTaskWaiting = false;
	}

	/**
	 * Closes this output stream and releases any system resources
	 * associated with this stream. The general contract of
	 * <code>close</code>
	 * is that it closes the output stream. A closed stream cannot
	 * perform output operations and cannot be reopened.
	 */
	public void close() {
		
		flush();
		hasBeenClosed = true;
		if (systemErr)
			System.setErr(org);
		else
			System.setOut(org);
	}


	/**
	 * Writes the specified byte to this output stream. The general
	 * contract for <code>write</code> is that one byte is written
	 * to the output stream. The byte to be written is the eight
	 * low-order bits of the argument <code>b</code>. The 24
	 * high-order bits of <code>b</code> are ignored.
	 *
	 * @param b the <code>byte</code> to write
	 * @throws IOException if an I/O error occurs. In particular,
	 *                     an <code>IOException</code> may be thrown if
	 * 					the output stream has been closed.
	 */
	public void write(final int b) throws IOException {
		
		if (hasBeenClosed) {
			throw new IOException("The stream has been closed.");
		}
		if (writeToOriginalStreamAlso)
			org.write(b);
		// would this be writing past the buffer?
		if (count == bufLength) {
			// then grow the buffer
			final int newBufLength = bufLength + DEFAULT_BUFFER_LENGTH;
			final byte[] newBuf = new byte[newBufLength];
			System.arraycopy(buf, 0, newBuf, 0, bufLength); 
			buf = newBuf;
			bufLength = newBufLength;
		}
		buf[count] = (byte) b;
		count++;
		if (!flushTaskWaiting && (count > 1)) {
			logtimer.schedule(new flushTask(), flushDelay);
			flushTaskWaiting = true;
		}
	}


	/**
	 * Flushes this output stream and forces any buffered output bytes
	 * to be written out. The general contract of <code>flush</code> is
	 * that calling it is an indication that, if any bytes previously
	 * written have been buffered by the implementation of the output
	 * stream, such bytes should immediately be written to their
	 * intended destination.
	 */
	public void flush() {

		if (count == 0) {
			return;
		}
//		don't print out blank lines; flushing from PrintStream puts
// 		out blank lines.
//		For linux system
		if (count == 1 && ((char) buf[0]) == '\n') {
			reset();
			return;
		}
//		For mac system
		if (count == 1 && ((char) buf[0]) == '\r') {
			reset();
			return;
		}
//		On windows system
		if (count==2 && (char)buf[0]=='\r' && (char)buf[1]=='\n') {
			reset();
			return;
		}
		final byte[] theBytes = new byte[count]; 
		System.arraycopy(buf, 0, theBytes, 0, count); 
		if (systemErr)
			Log.logError("JavaSystemError - " + new String(theBytes));
		else 
			Log.log("JavaSystemOut: " + new String(theBytes));
		reset();
	}

	private void flushQueued() {

		flush();
		flushTaskWaiting = false;
	}

	private void reset() {
		// not resetting the buffer -- assuming that if it grew then it
		//   will likely grow similarly again
		count = 0;
	}
	
	class flushTask extends TimerTask {
		public void run() {
			flushQueued();
		}
	}
}
