package net.java.dev.fwutil;

import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.table.TableRowSorter;

/** 
 * A MTable that listens for mouse-clicks in the mark-column. 
 * Utility functions markAll/Selected(true/false)/Inverse available and
 * actions markInverseRow, markSelected, unmarkSelected, markInverse and markAll
 * registered for keys space, ctrl + (shift) m, ctrl+i and ctrl+a.
 * Requires a typed MarkTableModel (T) to function properly.
 * @author Fred
 *
 */
public abstract class MarkTable<T> extends MTable {

	private static final long serialVersionUID = 7773185388425868238L;

	public MarkTable(String tableIdentiefier) {
		super(tableIdentiefier);
		setSelectionMode(javax.swing.ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		setAutoCreateRowSorter(false);
		addMarkActions();
	}
	
	/** Called by constructor. */
	@SuppressWarnings("serial")
	protected void addMarkActions() {
		
		Action markInverseRow = new AbstractAction(){
			public void actionPerformed(final ActionEvent ae){
				markInverseRow();
			}
		};
		getInputMap(JComponent.WHEN_FOCUSED).put(KeyStroke.getKeyStroke(KeyEvent.VK_SPACE, 0), "markInverseRow");
		getActionMap().put("markInverseRow", markInverseRow);

		Action markSelected = new AbstractAction(){
			public void actionPerformed(final ActionEvent ae){
				markSelected(true);
			}
		};
		getInputMap(JComponent.WHEN_FOCUSED).put(KeyStroke.getKeyStroke(KeyEvent.VK_M, InputEvent.CTRL_DOWN_MASK), "markSelected");
		getActionMap().put("markSelected", markSelected);

		Action unmarkSelected = new AbstractAction(){
			public void actionPerformed(final ActionEvent ae){
				markSelected(false);
			}
		};
		getInputMap(JComponent.WHEN_FOCUSED).put(KeyStroke.getKeyStroke(KeyEvent.VK_M, InputEvent.CTRL_DOWN_MASK | InputEvent.SHIFT_DOWN_MASK), "unmarkSelected");
		getActionMap().put("unmarkSelected", unmarkSelected);

		Action markInverse = new AbstractAction(){
			public void actionPerformed(final ActionEvent ae){
				markInverse();
			}
		};
		getInputMap(JComponent.WHEN_FOCUSED).put(KeyStroke.getKeyStroke(KeyEvent.VK_I, InputEvent.CTRL_DOWN_MASK), "markInverse");
		getActionMap().put("markInverse", markInverse);

		Action markAll = new AbstractAction(){
			public void actionPerformed(final ActionEvent ae){
				markAll();
			}
		};
		getInputMap(JComponent.WHEN_FOCUSED).put(KeyStroke.getKeyStroke(KeyEvent.VK_A, InputEvent.CTRL_DOWN_MASK), "markAll");
		getActionMap().put("markAll", markAll);

		Action toggleAutoSort = new AbstractAction(){
			public void actionPerformed(final ActionEvent ae){
				toggleAutoSort();
			}
		};
		getInputMap(JComponent.WHEN_FOCUSED).put(KeyStroke.getKeyStroke(KeyEvent.VK_U, InputEvent.CTRL_DOWN_MASK), "toggleAutoSort");
		getActionMap().put("toggleAutoSort", toggleAutoSort);
	}
	
	@Override
	public abstract MarkTableModel<T> getModel();
	
	/** 
	 * The default check-box cell-editor produces strange results when combining mouse clicks
	 * and then shortcut-keys (e.g. Inverse marks) and then navigation keys (e.g. up). 
	 * Work-around these problems by letting MarkTableModel.isCellEditable return false and
	 * overload this method to react to mouse-clicks on the Mark-column.
	 */
	@Override
	protected void processMouseEvent(MouseEvent e) {
		
		boolean consumed = false;
		if (e.getID() == MouseEvent.MOUSE_CLICKED && SwingUtilities.isLeftMouseButton(e) && e.getClickCount() % 2 == 1) {
			int columnClicked = columnAtPoint(e.getPoint());
			if (columnClicked > -1) columnClicked = convertColumnIndexToModel(columnClicked);
			if (columnClicked == getModel().getMarkColumn()) {
				int rowClicked = rowAtPoint(e.getPoint());
				if (rowClicked > -1) {
					rowClicked = convertRowIndexToModel(rowClicked);
					boolean currentValue = (Boolean)getModel().getValueAt(rowClicked, columnClicked) ;
					getModel().setValueAt(!currentValue, rowClicked, columnClicked);
					e.consume();
					consumed = true;
				}
			}
		}
		if (consumed) e.consume();
		else super.processMouseEvent(e);
	}
	
	public void markInverseRow() {

		SortOnUpdateLock slock = new SortOnUpdateLock();
		int rowIndex = getRowSorter().convertRowIndexToModel(getSelectedRow());
		getModel().setValueAt( !(Boolean)getModel().getValueAt(rowIndex, getModel().getMarkColumn()), 
				rowIndex, getModel().getMarkColumn());
		slock.unlock();
	}

	public void markAll() {

		SortOnUpdateLock slock = new SortOnUpdateLock();
		int i = 0;
		while (i < getModel().getRowCount()) {
			getModel().setValueAt(Boolean.TRUE, i, getModel().getMarkColumn());
			i++;
		}
		slock.unlock();
	}
	
	public void markSelected(boolean value) {

		SortOnUpdateLock slock = new SortOnUpdateLock();
		int[] rows = getSelectedRows();
		int i = 0;
		while (i < rows.length) {
			getModel().setValueAt(value, 
					getRowSorter().convertRowIndexToModel(rows[i]), getModel().getMarkColumn());
			i++;
		}
		slock.unlock();
	}
	
	public void markInverse() {
		
		SortOnUpdateLock slock = new SortOnUpdateLock();
		int i = 0;
		while (i < getModel().getRowCount()) {
			getModel().setValueAt(
					!((Boolean)getModel().getValueAt(i, getModel().getMarkColumn())),
					i, getModel().getMarkColumn());
			i++;
		}
		slock.unlock();
	}
	
	public boolean toggleAutoSort() {
		
		boolean autoSortOn = false;
		 TableRowSorter<?> rowSorter = (getRowSorter() == null ? null : 
				getRowSorter() instanceof TableRowSorter<?> ? (TableRowSorter<?>)getRowSorter() : null);
		 if (rowSorter != null) {
			 autoSortOn = !rowSorter.getSortsOnUpdates();
			 rowSorter.setSortsOnUpdates(autoSortOn);
			 if (autoSortOn) {
				 Log.log("Enabled row-sorting on value updates.");
			 } else {
				 Log.log("Disabled row-sorting on value updates.");
			 }
		 }
		 return autoSortOn;
	}
	
	public void refresh(boolean structureChanged) {

		if (getModel() != null) {
			getModel().syncMarked();
			if (structureChanged) {
				getModel().fireTableStructureChanged();
				applyUIProps();
			} else {
				getModel().fireTableDataChanged();
			}
		}
	}

	public class SortOnUpdateLock {
		
		public TableRowSorter<?> rowSorter = null;
		public boolean updateOnRowChange = false;
		public int selectedViewRow = -1;

		public SortOnUpdateLock() {
			super();
			rowSorter = (getRowSorter() == null ? null : 
				getRowSorter() instanceof TableRowSorter<?> ? (TableRowSorter<?>)getRowSorter() : null);
			if (rowSorter != null) {
				updateOnRowChange = rowSorter.getSortsOnUpdates();
				rowSorter.setSortsOnUpdates(false);
			}
			selectedViewRow = getSelectedRow();
			//Log.log("Selected row for view before update: " + selectedViewRow);
		}
		
		public void unlock() {
		
			if (rowSorter != null) rowSorter.setSortsOnUpdates(updateOnRowChange);
			if (updateOnRowChange && selectedViewRow > -1 && getModel() != null) {
				getModel().fireTableDataChanged();
				if (selectedViewRow >= getModel().getRowCount()) {
					selectedViewRow = getModel().getRowCount() - 1;
				}
				// Check again, model may have changed to 0 rows.
				if (selectedViewRow > -1) setRowSelectionInterval(selectedViewRow, selectedViewRow);
				//Log.log("Selected row for view: " + selectedViewRow);
			}
		}
	}
}
