package net.java.dev.fwutil;

import java.util.HashSet;
import java.util.Iterator;

import javax.swing.event.TableModelEvent;
import javax.swing.table.AbstractTableModel;

public abstract class MarkTableModel<T> extends AbstractTableModel {

	private static final long serialVersionUID = -6322826164087882623L;

	protected int markColumn = 0;
	
	protected HashSet<T> markedRows = new HashSet<T>();
	
	@SuppressWarnings("unchecked")
	public HashSet<T> getMarkedRowIdSet() {
		return (HashSet<T>)markedRows.clone();
	}

	public int getMarkColumn() {
		return markColumn;
	}
	
	@Override
	public Class<?> getColumnClass(final int col) {
		return (col == markColumn ? Boolean.class : super.getColumnClass(col));
	}
	
	/** Should return true for Mark-column but this gives change results
	 * when rows are selected using mouse and then shortcut-key (e.g. ctrl-c)
	 * MarkTable.processMouseEvent is overloaded to react to mouse-clicks on the Mark-column.
	 */
	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return (columnIndex == markColumn ? false : super.isCellEditable(rowIndex, columnIndex));
	}
	
	public abstract T getRowId(int row);
	public abstract int getRowIndex(T rowId);
	
	public void syncMarked() {
		
		HashSet<T> markedRowIds = getMarkedRowIdSet();
		final Iterator<T> markedIds = markedRowIds.iterator();
		final HashSet<T> zombies = new HashSet<T>();
		T rowId;
		while (markedIds.hasNext()) {
			rowId = markedIds.next();
			int rowIndex = getRowIndex(rowId);
			if (rowIndex < 0) zombies.add(rowId);
		}
		final Iterator<T> zombieIds = zombies.iterator();
		while (zombieIds.hasNext())
			markedRows.remove(zombieIds.next());
		int i = 0;
		while (i < getRowCount()) {
			rowId = getRowId(i);
			Boolean marked = getValueMarked(i);
			if (marked == null || !marked) markedRows.remove(rowId);
			else markedRows.add(rowId);
			i++;
		}
	}
	
	/** Overload to retrieve the marked value from source directly,
	 * not using getValueAt (which this method calls).
	 * @param rowIndex
	 * @return
	 */
	public Boolean getValueMarked(int rowIndex) {

		Object v = getValueAt(rowIndex, markColumn);
		Boolean b = (v == null ? null : Boolean.FALSE);
		if (b != null && v instanceof Boolean)
			b = (Boolean)v;
		return b;
	}

	public Object getValueAt(int rowIndex, int columnIndex) {
		return (columnIndex == markColumn ? markedRows.contains(getRowId(rowIndex)) : null);
	}
	
	public boolean tableRowsChanged(TableModelEvent e) {
		
		return (e.getFirstRow() == 0 && e.getLastRow() == Integer.MAX_VALUE
				&& e.getColumn() == TableModelEvent.ALL_COLUMNS);
	}
	
	@Override
	public void fireTableChanged(TableModelEvent e) {
		// Check if fireTableDataChanged() was called.
		if (tableRowsChanged(e)) syncMarked();
		super.fireTableChanged(e);
	}
	
	/** 
	 * Empty method which can be overloaded to take action when a mark is removed/added. 
	 * Called by setValueAt. */
	public void setMarked(int rowIndex, boolean marked) {
	}
	
	@Override
	public void setValueAt(Object value, int rowIndex, int columnIndex) {
		
		if (columnIndex == markColumn && value instanceof Boolean) {
			if (value == null) return; // Either false or true, but not null.
			final boolean v = (Boolean)value;
			final T rowId = getRowId(rowIndex);
			final boolean isMarked = markedRows.contains(rowId);
			if (v == isMarked) return;
			if (v) markedRows.add(rowId);
			else markedRows.remove(rowId);
			setMarked(rowIndex, v);
			fireTableCellUpdated(rowIndex, columnIndex);
		}
	}
}
