/*
 * This software is licensed under the GNU Lesser GPL, see
 * http://www.gnu.org/licenses/ for details.
 * You can use this free software in commercial and non-commercial products,
 * as long as you give credit to this software when you use it.
 * This softwware is provided "as is" and the use of this software is 
 * completely at your own risk, there are absolutely no warranties.
 * 
 */

package net.java.dev.fwutil;

import net.java.dev.hvlayout.*;

import java.awt.Cursor;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.KeyStroke;
import javax.swing.WindowConstants;

/**
 * A modal dialog that fires a cancel action when Escape is pressed or the window 
 * is closed. Furthermore, the default action is set to the "OK" button so that
 * pressing Enter fires an action for the OK button.
 * See the private init() method for how this is achieved.
 * <br> The Window dimensions are automatically stored and/or applied if
 * they are available. 
 * <br> See FontSelector on how this class can be used.
 * @author Fred
 *
 */
public class OkCancelDialog extends JDialog {
	
	private static final long serialVersionUID = 1L;
	private ActionListener listener;
	private JButton bOk;
	private JButton bCancel;

	/**
	 * Instantiates this dialog (without showing it).
	 * Use getOkButton() and getCancelButton() to detect OK and Cancel button actions.
	 * @param owner The parent frame.
	 * @param title Dialog title.
	 * @param okCancelListener The Actionlistener processing OK and Cancel actions 
	 */
	public OkCancelDialog(Frame owner, String title, ActionListener okCancelListener) {
		
		super(owner, title, true);
		// Java 5:
		// setModal(true);
		// Java 6:
		setModalityType(ModalityType.APPLICATION_MODAL);

		listener = okCancelListener;
		init();
	}
	
	/**
	 * Instantiates this dialog (without showing it).
	 * Use getOkButton() and getCancelButton() to detect OK and Cancel button actions.
	 * @param owner The parent frame.
	 * @param title Dialog title.
	 * @param okCancelListener The Actionlistener processing OK and Cancel actions 
	 */
	public OkCancelDialog(Dialog owner, String title, ActionListener okCancelListener) {
		
		super(owner, title, true);
		listener = okCancelListener;
		init();
	}

	private void init() {

		bOk = FormUI.getButton(listener, Labels.get("OK"));
    	bCancel = FormUI.getButton(listener, Labels.get("Cancel"));
		setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		addWindowListener(new WindowCloseAdapter(bCancel, listener));
		getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), bCancel);
		getRootPane().getActionMap().put(bCancel, new KeyAction(bCancel, listener));
		getRootPane().getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), bOk);
		getRootPane().getActionMap().put(bOk, new KeyAction(bOk, listener));
	}
	
	/**
	 * The OK-button is used as source for OK-actions.
	 */
	public JButton getOkButton() {
		return bOk;
	}
	
	/**
	 * The Cancel-button is used as source for Cancel-actions.
	 */
	public JButton getCancelButton() {
		return bCancel;
	}

	/**
	 * A container with the OK and Cancel buttons (aligned opposite the
	 * reading direction).
	 */
	public HBox getOkCancelLine() {
		
		HBox lb = new HBox();
		lb = new HBox(HBox.OPPOSITELINEAXIS);
		//lb = new LineBox();
		lb.add(bOk);
		lb.add(bCancel);
		return lb;
	}
	
	public void setWaitCursor() {
		getContentPane().setCursor(new Cursor(Cursor.WAIT_CURSOR));
	}
	
	public void setNormalCursor() {
		getContentPane().setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
	}


	/**
	 * Overloaded to use stored window dimensions for this dialog.
	 */
	public void pack() {
		super.pack();
		WindowDimension.applyWindowDimension("OkCancelDialog" + this.getTitle(), this, getOwner());
	}
	
	/**
	 * Overloaded to store window dimensions for this dialog.
	 */
	public void setVisible(boolean visible) {
		
		if (!visible)
			WindowDimension.setWindowDimension("OkCancelDialog" + this.getTitle(), this);
		super.setVisible(visible);
	}
	
	/**
	 * Overloaded to clear the OK and cancel button and the listener.
	 */
	public void dispose() {
		
		bOk = null;
		bCancel = null;
		listener = null;
		super.dispose();
	}

}
