/*
 * This software is licensed under the GNU Lesser GPL, see
 * http://www.gnu.org/licenses/ for details.
 * You can use this free software in commercial and non-commercial products,
 * as long as you give credit to this software when you use it.
 * This softwware is provided "as is" and the use of this software is 
 * completely at your own risk, there are absolutely no warranties.
 * 
 */

package net.java.dev.fwutil;

import java.beans.Encoder;
import java.beans.Expression;
import java.beans.DefaultPersistenceDelegate;
import javax.swing.RowSorter.SortKey;
import javax.swing.SortOrder;

/**
 * The XMLEncoder normally knows how to encode objects automatically IF
 * they follow the Java Beans formats (with a lot getters and setters).
 * An example of this is WindowDimension.
 * But SortKey uses SortOrder which is an Enum. That is something the
 * XMLEncoder did not like. So this PersistenceDelegate specifies how
 * to initiate a new SortKey using values from an old SortKey object. 
 * 
 * Persistence delegates only need to be registered once (they become meta-data)
 * using XMLEncoder.setPersistenceDelegate(javax.swing.RowSorter.SortKey, 
 * new SortKeyPersistenceDelegate());
 * From then on, the persistence delegate is known to the Java Runtime Environment,
 * even when the XMLEncoder object is no longer available.
 * XMLDecoder automatically re-uses the persistence delegate.
 */
public class SortKeyPersistenceDelegate extends DefaultPersistenceDelegate {

	/**
	 * This expression is called by the Encoder and used by the Decoder.
	 * The Encoder uses the expression to figure out which data 
	 * must be stored .
	 * The decoder recognizes the "new" keyword and knows to use 
	 * this expression upon object creation like 
	 * "new RowSorter.SortKey(int column, SortOrder sortOrder)".
	 * <p>
	 * Without this expression, the SortKey is created like "new SortKey()",
	 * resulting in an error. Also, the actual values of the SortKey object
	 * would not be stored (just the class name would be stored).
	 */
	protected Expression instantiate(Object oldInstance, Encoder out) {
		super.instantiate(oldInstance, out);
		
		/*
		 * To create a SortKey, two parameters are need, they are stored in args.
		 */
		Object[] args = new Object[2];
		
		/*
		 * First parameter is the column, so retrieve column.
		 */
		args[0] = new Integer(((SortKey) oldInstance).getColumn());
		// SortOrder is a type-specific Enumeration, so must return SortOrder value.
		args[1] = SortOrder.valueOf(((SortKey) oldInstance).getSortOrder().name());
		return new Expression(oldInstance, oldInstance.getClass(), "new", args);
	}
}
