/*
 * This software is licensed under the GNU Lesser GPL, see
 * http://www.gnu.org/licenses/ for details.
 * You can use this free software in commercial and non-commercial products,
 * as long as you give credit to this software when you use it.
 * This softwware is provided "as is" and the use of this software is 
 * completely at your own risk, there are absolutely no warranties.
 * 
 */

package net.java.dev.fwutil;

import java.util.Hashtable;

import javax.swing.LookAndFeel;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;


/**
 * Provides one method to initialize the application environment (boot) 
 * and to close it (close(0)).
 * Requires a label-file. 
 * 
 * Logging is started and the logging Window is shown to inidcate progress at booting.
 * Settings are loaded and/or initialized. Labels are loaded (@see Labels.java).
 */
public final class Startup {
	/* Requires labels:
	 * ReadSettings =Reading settings from file 
	 * NotReadSettings =Could not read settings from file
	 * 
	 * Uses window dimension LogFrame for logging window.
	 */
	
	/** Indicates if the first boot has been done. */
	public static boolean firstBoot = true;
	/** Indicates if a restart should be done. */
	public static boolean restart = false;
	/** Indicates if settings should be stored or not. */
	public static boolean doNotWriteSettings = false;

	private static String[] mainArgs;
	private static boolean closingDown = false;

	/**
	 * Initializes the application environment using classes provided 
	 * by the Util package. <br>
	 * 
	 * Commands are read from the command line: <br>
	 * -debug : sets Settings.appDebug. Default is off.<br>
	 * -labels=file : specifies the file to read labels from. Default is conf\labels.txt <br>
	 * <br> 
	 * Labels are read and processed.<br>
	 * Settings are set, loaded if possible and initialized (e.g the application data dir).<br>
	 * <br> 
	 * @param args command line arguments passed to the main method <br>
	 * @param appName the application name (for Settings).<br>
	 * @param appVersion the application version (for Settings)<br>
	 * @param dataDirName name for the data directory. If null is passed, the
	 * system temporary directory will become the data directory. Tip: start the
	 * name with a dot so that on Linux/Unix the data directory follows the convention
	 * for those systems.  
	 */
	public static boolean boot(String[] args, String appName, 
			String appVersion, String dataDirName) {
		
		if (restart)
			firstBoot = false;
		mainArgs = args;
		Settings.appName = appName;
		Settings.appVersion = appVersion;
		Settings.appDataDir = Util.getDataDir(dataDirName);

		Hashtable<String, String> params = Util.getArgs(args);
		if (params.size() > 0) {
			if (params.containsKey("debug")) { 
				Settings.appDebug = true;
			}
			if (params.containsKey("labels")) {
				Labels.labelFile=params.get("labels");
			}
		}
		/*
		 * Before the following statement, this method may not exit 
		 * or the program could end up in an infinite loop.
		 */
		restart = false;
		
		if (Settings.appDataDir == null) {
			System.out.println("Failed to open data directory: " + dataDirName);
			System.exit(1);
		}

		Settings.defaultSettingsFile = Settings.appDataDir + Util.fileSeparator + Settings.defaultSettingsFile; 
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				Log.makeVisible();
            }
        });
		if (!Log.initialize(Settings.appDataDir, Settings.appName + " " + Settings.appVersion + " starting ..."))
			return false;
		//Log.log(Settings.appName + " " + Settings.appVersion);

		/*
		 * Unfortunately the code below only works for catching exceptions in
		 * the main thread and not in thread "AWT-EventQueue-0". 
		 * Initialization errors thrown in this thread simply disappear.
		 * Other errors could disappear as well.
		 * 
		 *  LogJavaSystem provides a more robust solution (and is used in class Log,
		 *  allthough it cannot notify the user of errors.
		 *  But this could be extended.
		 */
		
		// if (!Settings.appDebug) 
		//	Thread.setDefaultUncaughtExceptionHandler(new UtilExceptionHandler());
		
		Log.log("Running on " + System.getProperty("os.arch") + Util.space +
				System.getProperty("os.name") + Util.space +
				System.getProperty("os.version") +	" with " + 
				System.getProperty("java.vm.vendor") + Util.space +
				System.getProperty("java.vm.name") + Util.space +
				System.getProperty("java.vm.version"));
		if (!Labels.initialize()) {
			Log.logError("Failed to load labels");
			return false;
		} else {
			Log.log("Labels loaded from: " + Labels.labelFile);
		}
		Log.log(Labels.get("ReadSettings") + Settings.defaultSettingsFile);
		if (!Settings.readSettings(Settings.defaultSettingsFile)) {
			Log.log(Labels.get("NotReadSettings"));
			if (Settings.appDebug)
				Log.log("First start detected");
			Settings.firstStart = true;
		} else {
			Settings.firstStart = false;
			Settings.setDefaultFont();
		}
		// Initialize parameters:
		WindowDimension.applyWindowDimension("LogFrame", LogUI.getInstance());

		// Following code causes a somewhat gracefull shutdown when application is killed
		Runtime.getRuntime().addShutdownHook(new Thread(new Runnable() { 
			public void run() {
				// Only shutdown when we are not already shutting down.
				// If this is not checked, close(0) will be called even after
				// System.exit which means the application never terminates.
				if (!closingDown)
					close(1);
			}
		}));
		return true;
	}
	
	/** Close the fwutil package and exit with exitCode (0 indicates OK). */
	public static void close(int exitCode) {
		close(exitCode, null);
	}
	
	/** If restart if true, closes and then calls the restartable. */
	public static void close(Restartable mainApp) {
		close(0, mainApp);
	}

	/** Closes the fwutil package (e.g. write settings, close log, clear memory)
	 * and if restart is true, calls the restartable.
	 */
	public static void close(int exitCode, Restartable mainApp) {

		closingDown = true;
//		if (Settings.appDebug)
//			System.out.println("Closing down");
		WindowDimension.setWindowDimension("LogFrame", LogUI.getInstance());
		Log.makeVisible();
		Log.log(Labels.get("WriteSettings") + Settings.defaultSettingsFile);
		if (!doNotWriteSettings && !Settings.writeSettings(Settings.defaultSettingsFile)) {
			if (Settings.appDebug)
				System.out.println("Failed to write settings!");
			Log.logError(Labels.get("WriteSettingsFailed") + Util.newline + Log.getLogFileName());
		}
		Log.log(Labels.get("Finished"));
		if (Startup.restart && mainApp == null)
			Log.logError("StartUp.close: Cannot restart because mainApp is unknown");
		Log.close();
		Labels.clear();
		Settings.clearSettings();
		Startup.firstBoot = false;
/*		if (StartUp.restart)
			System.out.println("Restart true.");
		if (mainApp != null)
			System.out.println("MainApp exists.");
*/		if (Startup.restart && mainApp != null) {
			closingDown = false;
			/*
			 * See comments on Settings.setDefaultFont:
			 * this is an attempt to get the default system fonts back.
			 * Unfortunately this does not work properly.
			 */
			try {
				UIManager.setLookAndFeel((LookAndFeel)(null));
			} catch (UnsupportedLookAndFeelException e) {
			}
			try {
				UIManager.setLookAndFeel(
						UIManager.getSystemLookAndFeelClassName());
			} catch (Exception unused) { }
			mainApp.restart(mainArgs);
		} else {
			if (Settings.appDebug)
				System.out.println("Exiting with " + exitCode);
			System.exit(exitCode);
		}
	}
}
