package net.java.dev.fwutil;

import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

/** A listMap that uses Strings as keys. 
 * Provides to convenience options:
 *  - caseSensitivity switch for keys
 *  - options to get all keys that start with a given String (see getKeys(rootKey)). 
 *  */
public class StringListMap<V> extends ListMap<String, V> {

	private static final long serialVersionUID = -4815228677564977195L;
	
	/** Set to false to make keys case-insensitive. */
	public boolean caseSensitive = true;

	/** Returns the key found that matches the given key while ignoring case for the keys
	 * if caseSensitive is false. */
	public String getProperCaseKey(final String key) {
		
		String keyPC = key;
		if (key != null && !caseSensitive) {
			Iterator<String> keys = keySet().iterator();
			String k = null;
			while (keyPC != k && keys.hasNext()) {
				k = keys.next();
				if (key.equalsIgnoreCase(k)) {
					keyPC = k;
				}
			}
		}
		return keyPC; 
	}
	
	public List<V> get(String key) {
		return super.get(getProperCaseKey(key));
	}
	
	/** Returns all keys that start with rootKey (rootKey can be null). */
	public Set<String> getKeys(final String rootKey) {
		
		Set<String> keys = new HashSet<String>();
		Iterator<String> it = keySet().iterator();
		final String rootKeyUpper = (rootKey == null ? null : rootKey.toUpperCase());
		while (it.hasNext()) {
			String key = it.next();
			if (key == null && rootKey == null) {
				keys.add(key);
			} else if (key != null ) {
				if (caseSensitive && key.startsWith(rootKey)) {
					keys.add(key);
				} else if (!caseSensitive && key.toUpperCase().startsWith(rootKeyUpper)) {
					keys.add(key);
				}
			}
		}
		return keys;
	}

	public boolean containsValue(final String key, final V value) {
		return super.containsValue(getProperCaseKey(key), value);
	}

	public void set(final String key, final V value) {
		super.set(getProperCaseKey(key), value);
	}

	public void add(final String key, final V value) {
		super.add(getProperCaseKey(key), value);
	}
	
	public List<V> remove(final String key) {
		return super.remove(getProperCaseKey(key));
	}

	public boolean remove(final String key, final V value) {
		return super.remove(getProperCaseKey(key), value);
	}
}
