package net.java.dev.fwutil.gui;

import java.awt.Component;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.lang.reflect.Constructor;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JMenuBar;
import javax.swing.JSeparator;

import net.java.dev.fwutil.Labels;
import net.java.dev.fwutil.Log;
import net.java.dev.fwutil.LogUI;
import net.java.dev.fwutil.MFrame;
import net.java.dev.fwutil.Restartable;
import net.java.dev.fwutil.Settings;
import net.java.dev.fwutil.Startup;
import net.java.dev.fwutil.UIMessages;
import net.java.dev.fwutil.Util;
import net.java.dev.hvlayout.FormUI;
import net.java.dev.hvlayout.HVProps;
import net.java.dev.hvlayout.VBox;

/** Basic Main Window with statusbar and menu. 
 * Use openWindow to initiate this class from the main method.
 * Override */
public class BMainWindow extends MFrame implements IMainWindow {

	private static final long serialVersionUID = -6342150019014190700L;

	protected static boolean exitingApp = false;
	protected VBox mainPanel;
	protected IMainMenu imainMenu;
	protected BMainStatusBar statusBar;
	protected Restartable mainRestart;

	public BMainWindow(Restartable mainRestart, Component owner) {

		// Always set LogUI as parent so that File> Exit works (which calls LogUI.close).
		super(LogUI.getInstance(), Settings.appName + " " + Settings.appVersion);
		this.mainRestart = mainRestart;
		mainPanel = new VBox(null);
		setContentPane(mainPanel);
		statusBar = null;
		buildWindow();
		if (LogUI.openChildWindows() == 0) {
			LogUI.setProgrammaticClose();
			if (!Settings.appDebug) Log.makeInvisible(); 
		}
		// Applies windows dimensions.
		display(owner);
		if (statusBar != null)
			statusBar.setText(Labels.get("Ready"));
	}
	
	public static void openWindow(final Class<?> clazz, final Restartable mainRestart, 
			final Component owner) {
		
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				try {
					Constructor<?> con = clazz.getConstructor(Restartable.class, Component.class);
					con.newInstance(mainRestart, owner);
				} catch (Exception e) {
					Log.logError("Failed to open window " + Settings.appName + " " + Settings.appVersion);
					Log.logError(e);
				}
			}
		});
	}
	
	/** Override to set your own menu and window contents. 
	 * Fill the VBox mainPanel with required components. 
	 * Calls addStatusBar and sets imainMenu.
	 * Tries to load and set a window icon.
	 */
	protected void buildWindow() {

		Component filler = FormUI.getCanvasLine();
		filler.setMaximumSize(HVProps.screenSize);
		mainPanel.add(filler);
		addStatusBar();
		imainMenu = new BMainMenu(this);
		setJMenuBar((JMenuBar)imainMenu);
		loadWindowIcon(this, "resources/fw.png");
	}
	
	/** Adds the default statusbar to the mainPanel. */
	protected void addStatusBar() {
		
		JSeparator separator = new JSeparator();
		HVProps.setSize(separator, 0, separator.getPreferredSize().height, false);
		HVProps.doNotGrowHeigth(separator);
		mainPanel.add(separator);
		statusBar = new BMainStatusBar();
		statusBar.setText(Labels.get("Loading"));
		mainPanel.add(statusBar);
	}
	
	public Restartable getMainRestart() {
		return mainRestart;
	}
	
	/** Utility method to load images that can be used as icons. */
	public static boolean loadWindowIcon(final JFrame window, final String resourceName) {
		
		InputStream in = null;
		ByteArrayOutputStream bout = null;
		boolean ok = true;
		try {
			in = Util.getFileStream(resourceName);
			bout = new ByteArrayOutputStream();
			Util.copyStreams(in, bout);
			window.setIconImage(new ImageIcon(bout.toByteArray()).getImage());
		} catch (Exception e) {
			Log.logError("Failed to load main window icon: " + e);
			ok = false;
		} finally {
			try { in.close(); } catch (Exception ignored) {}
		}
		return ok;
	}
	
	/** 
	 * Called by dispose() and MWindow>Close Window, does not close the window if it returns false.
	 * Override to save files before exiting for example. 
	 */
	public boolean closeAllowed() {
		
		boolean allowed = true;
		if (LogUI.openChildWindows() == 1) {
			allowed = (UIMessages.askYesNo(this, 
				(Startup.restart ? Labels.get(BMLabel.Restart) : Labels.get(BMLabel.ExitApp)),
				Labels.get("ReallyExitApp")));
		} else {
			allowed = UIMessages.askYesNo(this, Labels.get(BMLabel.CloseWindow), Labels.get(BMLabel.CloseWindow) + "?");
		}
		return allowed; 
	}
	
	/**
	 * This method closes the application after storing all appropriate
	 * information if this is the last window that was closed.
	 * Calls closeAllowed() to check if windows can be closed. 
	 *
	 */
	public void exitApp(boolean restart) {

		exitingApp = true;
		Startup.restart = restart;
		int i = LogUI.openChildWindows();
		while(i > 0 &&
				((BMainWindow)LogUI.getChildWindows().get(i-1)).closeAllowed()) {
			LogUI.getChildWindows().get(i-1).dispose();
			i = LogUI.openChildWindows();
		}
		if (i == 0) {
			if (imainMenu.isClearSettingsSelected()) {
				if (!Util.deleteFile(Settings.defaultSettingsFile))
					Log.logError("Failed to delete settings file: " +
							Settings.defaultSettingsFile);
				else
					Log.log("Settings file deleted: " +
							Settings.defaultSettingsFile);
				// By default, StartUp.close() will write the Settings in a
				// setting-file. 
				Startup.doNotWriteSettings = true;
			}
			LogUI.close();
			Startup.close(getMainRestart());
		}
		// In case of restart.
		exitingApp = false;
	}
	
	/**
	 * Closes this window and releases all resources. 
	 * Checks if this is the last window closing,
	 * if so, and "Exit app?" has not been asked yet, that question will show first.
	 * Called by LogUI.close() and when window is closed by user clicking on X-button.
	 */
	public void dispose() {
		
		if (!exitingApp && LogUI.openChildWindows() == 1) {
			exitApp(false);
		} else {
			super.dispose();
		}
	}

}
