/*
 * This software is licensed under the GNU Lesser GPL, see
 * http://www.gnu.org/licenses/ for details.
 * You can use this free software in commercial and non-commercial products,
 * as long as you give credit to this software when you use it.
 * This softwware is provided "as is" and the use of this software is 
 * completely at your own risk, there are absolutely no warranties.
 * 
 */

package net.java.dev.hvlayout;

import java.awt.ComponentOrientation;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.UIManager;
import javax.swing.WindowConstants;

/**
 * Demo form to demonstrate HVLayout. You can run this example.
 * See {@code http://weblogs.java.net/blog/joconner/archive/2006/10/more_informatio.html}
 * for the intention of this form.
 * <br> This example does intentionally not use FormUI, HBox and VBox. 
 * It would be easier to use them, but this example shows that it is not difficult 
 * to do without them. 
 * <br> This form is NOT a good example of how to design a GUI, but it does
 * provide a baseline to compare this layout manager with
 * other layout managers. 
 * <br> Note that this implementation is a bit awkward due to the constraints given
 * by John O'Conner. A (better) re-designed form would be easier to implement.
 * E.g. to accomodate a proper tab-order, some components are lined out in one
 * horizontal container where using two horizontal containers would have been easier.
 */
@SuppressWarnings("serial")
public class AddressBookDemo extends JFrame {
	
	public static final String names[] = {"Bunny, Bugs", "Cat, Sylvester", 
			"Coyote, Wile", "Devil, Tasmanian", "Duck, Daffy", "Fudd, Elmer",
			"Le Pew, Pep", "Martian, Marvin"};
	/**
	 * Set to false to use the font set by fontName, fontStyle and fontSize.
	 * And see how HVLayout adapts to it.
	 */
	public static final boolean useSystemFont = true;
	public static final String fontName = "Arial";
	public static final int fontStyle = Font.PLAIN;
	public static final int fontSize = 20;
	
	public AddressBookDemo(String title) {
	
		super(title);
		setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
		if (!useSystemFont)
			HVProps.setDefaultUIFont(new Font(fontName, fontStyle, fontSize));
		// The main content pane for the frame.
		final Container mainPanel = this.getContentPane();
		// The main pane will put each component next to the other.
		mainPanel.setLayout(new HLayout());
		// The list with names shown on the left
		final JList nameList = new JList(names);
		// HVProps.defaultLineHeight scales with the font used on screen,
		// so it can also be used to set a width.JScrollPane listScroller
		final JScrollPane listScroller = HVProps.getScroller(nameList, HVProps.defaultLineHeight * 6,
					names.length, false); 
		mainPanel.add(listScroller);
		listScroller.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		HVProps.doNotShrinkWidth(listScroller);
		nameList.setSelectedIndex(names.length - 1);
		nameList.ensureIndexIsVisible(names.length - 1);
		
		// The container with the Last name, Phone, etc.
		final Container rightSide = getVBox();
		
		// Shows up to the right of the names list.
		mainPanel.add(rightSide);
		
		rightSide.add(getTextLine("Last name", "Martian", "First name", "Marvin"));
		rightSide.add(getTextLine("Phone", "+MARS-020-1010101", "Email", "masterblaster@redrock.lt"));
		rightSide.add(getTextLine("Address 1", "Dust Drive", null, null));
		// Alternative to Address 1: Street and housenumber.
		rightSide.add(getStreetNumber());
		rightSide.add(getTextLine("Address 2", "", null, null));
		rightSide.add(getTextLine("City", "Red Rock", null, null));
		rightSide.add(getTextLine("State", "Looney", "Postal Code", "ZZY BRBR"));
		rightSide.add(getTextLine("Country", "Looney Land", null, null));
		// Add canvas line so to add some space between the input fields
		// and the buttons.
		rightSide.add(HVProps.getCanvasLine(false));
				
		// A box that centers the buttons on screen.
		final Container buttons = getHBoxCentered();
		rightSide.add(buttons);
		
		// Play with font sizes to show how HVLayout accomodates this.
		buttons.add(getButton("New", 0));
		buttons.add(getButton("Delete", 4));
		buttons.add(getButton("Edit", 8));
		buttons.add(getButton("Save", 4));
		buttons.add(getButton("Cancel", 0));

		pack();
	}
	
	private Container getHBox() {
		
		final Container c = new Container();
		c.setLayout(new HLayout());
		return c;
	}
	
	private Container getHBoxCentered() {
		
		final Container c = new Container();
		c.setLayout(new HLayout(false, true));
		return c;
	}

	private Container getVBox() {
		
		final Container c = new Container();
		c.setLayout(new VLayout());
		return c;
	}
	
	/**
	 * Returns a |label:textField| line
	 */
	private Container getTextLine(final String label, final String text) {
		
		final Container lb = getHBox();
		lb.add(getLabel(label, true));
		lb.add(getTextField(text));
		return lb;
	}

	private Container getTextLine(final String label, final String text, 
			final String label2, final String text2) {
		
		final Container mainlb = getHBox();
		final Container lb = getTextLine(label, text);
		mainlb.add(lb);
		if (label2 == null)
			mainlb.add(HVProps.getEmptyLine(lb));
		else
			mainlb.add(getTextLine(label2, text2));
		return mainlb;
	}
	
	private JLabel getLabel(final String text, final boolean lineOutOppositeLineAxis) {
		
		final JLabel label = new JLabel(text);
		applyOrientation(label, lineOutOppositeLineAxis);
		// Sets a fixed size for the label, uses FormLayout.defaultButtonWidth
		// as default width.
		HVProps.setSize(label, 0, 0, true);
		// Normally, labels have a fixed size and the next
		// statement would not be necessary. But for the demo
		// labels can shrink in size too.
		setMinimumWidth(label);
		return label;
	}

	private JTextField getTextField(final String value) {

		final JTextField textLine = new JTextField(value);
		HVProps.setSize(textLine, HVProps.defaultLineWidth, 0, false);
		// Allow textfield to shrink some more (FormLayout sets minimum size
		// to FormLayout.defaultButtonWidth by default).
		setMinimumWidth(textLine);
		return textLine;
	}
	
	/**
	 * Returns a button with the choosen font size.
	 */
	private JButton getButton(final String text, final int fontSizeDelta) {
		
		final JButton button = new JButton(text);
		if (fontSizeDelta != 0) {
			final Font bfont = button.getFont();
			button.setFont(new Font(bfont.getName(), bfont.getStyle(), 
				bfont.getSize() + fontSizeDelta));
		}
		// The zeros default to FormLayout.defaultButtonWidth and
		// FormLayout defaultLineHeight
		HVProps.setSize(button, 0, 0, true);
		// Normally, buttons have a fixed size and the next
		// statement would not be necessary. But for the demo
		// buttons can shrink in size too.
		setMinimumWidth(button);
		return button;
	}
	
	/**
	 * Arbitrary choosen minimum-width: half the size of defaultButtonWidth
	 */
	private int getMinimumWidth() {
		return HVProps.defaultButtonWidth / 2;
	}
	
	/**
	 * Allow the component to shrink some more in width by 
	 * setting the minimum size smaller then the HVProps default.
	 */
	private void setMinimumWidth(final JComponent c) {
		c.setMinimumSize(new Dimension(getMinimumWidth(), c.getMinimumSize().height));
	}
	
	/**
	 * Just return a relative small value for width.
	 */
	private int getSmallWidth() {
		return HVProps.defaultLineHeight * 2;
	}
	
	/**
	 * Outlines a label to the right or not.
	 */
	private void applyOrientation(final JComponent c, final boolean lineOutOppositeLineAxis) {
		
		if (lineOutOppositeLineAxis) {
			if (c.getComponentOrientation().isLeftToRight()) {
				c.applyComponentOrientation(ComponentOrientation.RIGHT_TO_LEFT);
			} else {
				c.applyComponentOrientation(ComponentOrientation.LEFT_TO_RIGHT);
			}
		}
	}
	
	/**
	 * An alternative to the Address 1 line with street and house number
	 * 
	 * Street : ... : House# : ... :
	 * 
	 * Demonstrates calculating and settings a correct
	 * preferred size so that alignment stays in place
	 * even when the window is resized.
	 * @return
	 */
	private Container getStreetNumber() {
		
		final JLabel lstreet = getLabel("Street", true);
		final JTextField tstreet = getTextField("Dust Drive");
		
		final JLabel lhouse = getLabel("House#", true);
		final JTextField thouse = getTextField("42");
		final Container leftline = getHBox();
		leftline.add(lstreet);
		leftline.add(tstreet);
		leftline.add(lhouse);
		leftline.add(thouse);
		
		// Allow to shrink to nothing.
		setSmallToNullWidth(lhouse);
		setSmallToNullWidth(thouse);
		
		/* This text line is 2 * getSmallWidth() wider then a normal textline
		 * due to lhouse and thouse.
		 * Only tstreet can make up for this extra preferred width 
		 * when the window is shrinking.
		 * So we shrink tstreet by the extra added preferred width to prevent it
		 * from growing the width of the textlines. Comment/remove the statement
		 * to see the effect on the layout.
		 * HVProps.defaultLineWidth is the default preferred width for
		 * textfields, see getTextField.
		 */ 
		tstreet.setPreferredSize(new Dimension(
				HVProps.defaultLineWidth - 2 * getSmallWidth(),
				tstreet.getPreferredSize().height));
		final Container line = getHBox();
		line.add(leftline);
		line.add(HVProps.getEmptyLine(leftline));
		return line;
	}
	
	// Allow component to shrink to 0 width.
	private void setSmallToNullWidth(final JComponent c) {
		c.setMinimumSize(new Dimension(0, c.getMinimumSize().height));
		c.setPreferredSize(new Dimension(getSmallWidth(), c.getMinimumSize().height));
	}

	public static void main(String args[]) {
		
		try {
			UIManager.setLookAndFeel(
					UIManager.getSystemLookAndFeelClassName());
		} catch (Exception unused) { }

		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				final AddressBookDemo screen = new AddressBookDemo("Address Book Demo");
				screen.setVisible(true);
			}
		});
	}
}