/*
 * This software is licensed under the GNU Lesser GPL, see
 * http://www.gnu.org/licenses/ for details.
 * You can use this free software in commercial and non-commercial products,
 * as long as you give credit to this software when you use it.
 * This softwware is provided "as is" and the use of this software is 
 * completely at your own risk, there are absolutely no warranties.
 * 
 */

package net.java.dev.hvlayout;

import java.awt.Canvas;
import java.awt.Component;
import java.awt.ComponentOrientation;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Image;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.ImageIcon;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JPasswordField;
import javax.swing.JButton;
import javax.swing.JTree;
import javax.swing.event.TableModelListener;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.TreeSelectionModel;

import java.awt.event.MouseListener;

/**
 * This singleton class contains utility methods that use HVLayout 
 * to layout components and apply some tweaks to the components like tooltips.
 * Copy and paste what you like, or just use the methods in this class if you like
 * how the components behave.
 * @author Fred
 *
 */
public final class FormUI {

	public static Dimension fileChooseDialogSize = new Dimension(450, 350);
	
	private static FormUI thisInstance = new FormUI();

	private FormUI() {
		super();
	}
	
	/**
	 * An empty line, can be use for spacing vertically.
	 * Horizontal size is set to button width.
	 */
	public static Canvas getCanvasLine() {
		
		final Canvas cv = new Canvas();
		HVProps.setSize(cv, 0, 0, true);
		return cv;
	}

	/**
	 * An empty line, can be use for spacing vertically.
	 * Horizontal size is set to the given width.
	 */
	public static Canvas getCanvasLine(final int width) {
		
		final Canvas cv = new Canvas();
		HVProps.setSize(cv, width, 0, true);
		return cv;
	}
	
	/**
	 * A label that shows a tooltip when needed.
	 * If width is 0, defaultButtonWidth is used.
	 */
	public static JLabel getLabel(final String labelText, final int width) {

		return getLabel(labelText, width, false);
	}

	/**
	 * A label that shows a tooltip when needed.
	 * If lineOutOppositeLineAxis is true, label-text will show on the right side.
	 * If width is 0, defaultButtonWidth is used.
	 */
	public static JLabel getLabel(final String labelText, final int width, 
			final boolean lineOutOppositeLineAxis) {

		return getLabel(labelText, width, lineOutOppositeLineAxis, false, null);
	}

	/**
	 * A label that shows a tooltip when needed.
	 * If lineOutOppositeLineAxis is true, label-text will show on the right side.
	 * If width is 0, defaultButtonWidth is used.
	 * if fixedWidth is true, then the label will not grow beyond preferred width.
	 */
	public static JLabel getLabel(final String labelText, final int width, 
			final boolean lineOutOppositeLineAxis, final boolean fixedWidth) {
		return getLabel(labelText, width, lineOutOppositeLineAxis, fixedWidth, null);
	}

	/**
	 * A label that uses the given font and shows a tooltip when needed.
	 * If lineOutOppositeLineAxis is true, label-text will show on the right side.
	 * If width is 0, defaultLineWidth is used.
	 * if fixedWidth is true, then the label will not grow beyond preferred width.
	 * The label's height is adjusted for the default font and if width is 0,
	 * the width is adjusted as well.
	 */
	public static JLabel getLabel(final String labelText, int width, 
			final boolean lineOutOppositeLineAxis, final boolean fixedWidth, 
			final Font font) {

		final JLabel label = new HVLabel(labelText, font);
		if (width == 0 || width == HVProps.defaultLineWidth) {
			width = Math.max(HVProps.defaultLineWidth,
					label.getFontMetrics(label.getFont()).stringWidth(labelText));
		}
		HVProps.setSize(label, width, HVProps.defaultLineHeight, fixedWidth);
		if (lineOutOppositeLineAxis) {
			if (label.getComponentOrientation().isLeftToRight()) {
				label.applyComponentOrientation(ComponentOrientation.RIGHT_TO_LEFT);
			} else {
				label.applyComponentOrientation(ComponentOrientation.LEFT_TO_RIGHT);
			}
		}
		return label;
	}

	/* Should not be used, use getTextArea instead. Kept here for reference.
	
	public static VBox getMultiLineLabel(String labelText, int width) {

		return getMultiLineLabel(labelText, width, false, null);
	}

	public static VBox getMultiLineLabel(String labelText, int width, boolean lineOutOppositeLineAxis, Font font) {

		VBox box = new VBox(null);
		if (width == 0)
			width = HVProps.defaultLineWidth;
		StringLine.FileStringIterator fsi = StringLine.getNewFSI(labelText, 0);
		String line = StringLine.getNextLine(fsi);
		JLabel label = null;
		while (line != null) {
			label = getLabel(line, width, lineOutOppositeLineAxis, false, font);
			HVProps.setSize(label, label.getPreferredSize().width, HVProps.defaultLineHeight - HVProps.smallHeightCorrection, false);
			box.add(label);
			line = StringLine.getNextLine(fsi);
		}
		return box;
	}
	*/

	/**
	 * A button that shows a tooltip when needed.
	 */
	public static JButton getButton(final String label) {
		final JButton button = new HVButton(label);
		return button;
	}
	
	/**
	 * A button that shows a tooltip when needed.
	 */
	public static JButton getButton(final ActionListener al, final String label) {
		final JButton button = getButton(label);
		button.setActionCommand(label);
		button.addActionListener(al);
		return button;
	}

	/**
	 * Returns a square button with the specified icon and actionlistener. 
	 * The tooltip for the button is set to the description of the icon. 
	 */
	public static JButton getIconButton(ActionListener al, ImageIcon icon) {
		
		final JButton b = new HVButton(icon);
		b.addActionListener(al);
		return b;
	}
	
	public static ImageIcon getScaledIcon(final ImageIcon original) {
		
		Image newImage = original.getImage().getScaledInstance(HVProps.defaultLineHeight, HVProps.defaultLineHeight, Image.SCALE_SMOOTH);
		ImageIcon newImageIcon = new ImageIcon(newImage);
	       
        return newImageIcon;
	}

	/**
	 * A narrower button that shows a tooltip when needed.
	 */
	public static JButton getSmallButton(final String label) {
		final JButton button = new HVButton(label);
		HVProps.setSize(button, (int)(HVProps.defaultLineHeight * 2), 0, true);
		return button;
	}

	/**
	 * A wider button that shows a tooltip when needed.
	 */
	public static JButton getWideButton(final String label) {
		final JButton button = new HVButton(label);
		HVProps.setSize(button, HVProps.defaultButtonWidth * 2, 0, true);
		return button;
	}

	/**
	 * A checkbox that shows a tooltip when needed.
	 */
	public static JCheckBox getCheckBox(final ActionListener al, final String label, 
			final int width) {
		final JCheckBox cb = getCheckBox(label, width);
		cb.setActionCommand(label);
		cb.addActionListener(al);
		return cb;
	}

	@SuppressWarnings("serial")
	/**
	 * A checkbox that shows a tooltip when needed.
	 */
	public static JCheckBox getCheckBox(final String label, int width) {
		final JCheckBox cb = new JCheckBox(label) {
			public String getToolTipText(MouseEvent e) {
				FontMetrics fm = getFontMetrics(getFont());
				String text = getText();
				int textWidth = fm.stringWidth(text);
				return (textWidth > getSize().width ? text : null);
			}
		};
		cb.setToolTipText(" ");
		cb.setRequestFocusEnabled(true);
		if (width == 0)
			width = HVProps.defaultLineWidth;
		HVProps.setSize(cb, width, HVProps.defaultLineHeight - HVProps.smallHeightCorrection, false);
		return cb;
	}

	/**
	 * Gets a non-editable text area with a mouse-listener that
	 * switches WrapStyleWord (default on). 
	 * @param text The text to show.
	 * @param monospacedFont Set to true for a message area like log. Set to false for label-like text in a window 
	 * @return A text area with possibly a tooltip
	 */
	public static HVTextArea getTextArea(final String text, final boolean monospacedFont) {
		
		if (monospacedFont)
			return getTextArea(text, new Font(Font.MONOSPACED, Font.PLAIN, HVProps.getDefaultFontSize()));
		return getTextArea(text, null);
	}

	/**
	 * Creates a text area that is not editable and (un)wraps lines when clicked.
	 * Text selection is still possible with the mouse. A tooltip will
	 * appear when no monospace font is used.
	 * If the font is monospaced, the backrgound is white.
	 * <br> setEnabled(false) should not be used alone on TextArea,
	 * it makes the text difficult to read.
	 * @param text The text to show
	 * @param textFont The font used to show the text, null defaults to HVProps.getDefaultUIFont()
	 * @return A text area with possibly a tooltip
	 */
	public static HVTextArea getTextArea(final String text, Font textFont) {
		
		final HVTextArea textArea = new HVTextArea(text);
		textArea.setEditable(false);
		textArea.setLineWrap(false);
		textArea.setWrapStyleWord(true);
		textArea.addMouseListener(thisInstance.new MessageAreaWrapLinesListener());
		if (textFont == null)
			textFont = HVProps.getDefaultUIFont();
		textArea.setFont(textFont);
		// Font.MONOSPACED is a logical font, so use getName and not getFontName
		if (!textFont.getName().equals(Font.MONOSPACED)) {
			textArea.setOpaque(false);
			textArea.setToolTipText(text);
		} // else MONOSPACED font.
		return textArea;
	}
	
	/**
	 * This class is used by getTextArea() to turn on or off wrapping
	 * whith one nmouse click.
	 */
	public class MessageAreaWrapLinesListener implements MouseListener {
		public void mouseClicked(final MouseEvent event) {
			if (event.getButton() == MouseEvent.BUTTON1 && event.getSource() instanceof JTextArea) {
				final JTextArea source = (JTextArea) event.getSource();
				source.setLineWrap(!source.getLineWrap());
			}
		}
		public void mousePressed(final MouseEvent arg0) {}
		public void mouseReleased(final MouseEvent arg0) {}
		public void mouseEntered(final MouseEvent arg0) {}
		public void mouseExited(final MouseEvent arg0) {}
	}

	/**
	 * A text field.
	 * If width is 0, width is set to defaultLineWidth.
	 */
	public static JTextField getTextField(final ActionListener al, final String actionCommand, 
			final String defaultValue, final int width) {
		final JTextField textLine = getTextField(defaultValue, width);
		textLine.setActionCommand(actionCommand);
		textLine.addActionListener(al);
		return textLine;
	}

	/**
	 * A text field.
	 * If width is 0, width is set to defaultLineWidth.
	 */
	public static JTextField getTextField(final String defaultValue, int width) {
		final JTextField textLine = new JTextField(defaultValue);
		textLine.setRequestFocusEnabled(true);
		if (width == 0)
			width = HVProps.defaultLineWidth;
		HVProps.setSize(textLine, width, HVProps.defaultLineHeight, false);
		return textLine;
	}

	/**
	 * A password field.
	 * If width is 0, width is set to defaultLineWidth.
	 */
	public static JPasswordField getPasswordField(final ActionListener al, 
			final String actionCommand, final String defaultValue, 
			final int width) {
		final JPasswordField textLine = getPasswordField(defaultValue, width);
		textLine.setActionCommand(actionCommand);
		textLine.addActionListener(al);
		return textLine;
	}

	/**
	 * A password field.
	 * If width is 0, width is set to defaultLineWidth.
	 */
	public static JPasswordField getPasswordField(final String defaultValue, int width) {
		final JPasswordField textLine = new JPasswordField(defaultValue);
		textLine.setRequestFocusEnabled(true);
		if (width == 0)
			width = HVProps.defaultButtonWidth;
		HVProps.setSize(textLine, width, HVProps.defaultLineHeight, false);
		return textLine;
	}

	/**
	 * A list inside a scroller.
	 */
	public static Component getList(final JList list, final int width, 
			final int lines) {
		// No tweaks for list at this moment.
		return HVProps.getScroller(list, width, lines, false);
	}
	
	/**
	 * A table inside a scroller.
	 * Row-height is tweaked to fit the default font. 
	 */
	public static JScrollPane getTable(final JTable table, final int width, 
			final int lines, final TableModelListener tml) {

		table.setTableHeader(new HVTableHeader(table.getColumnModel()));
		table.setRowHeight(HVProps.defaultLineHeight - HVProps.smallHeightCorrection);
		table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
		table.getModel().addTableModelListener(tml);
		return HVProps.getScroller(table, width, lines, false);
	}

	/**
	 * A tree inside a scroller.
	 * Row-height is tweaked to fit the default font. 
	 */
	public static JScrollPane getTree(final JTree tree, final int width, 
			final int lines, final TreeSelectionListener tsl) {

		tree.setShowsRootHandles(true);
		tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		tree.setRowHeight(HVProps.defaultLineHeight - HVProps.smallHeightCorrection);
		tree.addTreeSelectionListener(tsl);
		return HVProps.getScroller(tree, width, lines, false);
	}
}
