/*
 * This software is licensed under the GNU Lesser GPL, see
 * http://www.gnu.org/licenses/ for details.
 * You can use this free software in commercial and non-commercial products,
 * as long as you give credit to this software when you use it.
 * This softwware is provided "as is" and the use of this software is 
 * completely at your own risk, there are absolutely no warranties.
 * 
 */

package net.java.dev.hvlayout;

import java.awt.Component;
import java.awt.ComponentOrientation;
import java.awt.Dimension;
import java.awt.Rectangle;

import javax.swing.JComponent;
import javax.swing.Scrollable;

/**
 * A container that lays out components horizontally. This container
 * can be placed inside a scroller. 
 * Use the appropriate constructor methods to layout components
 * left to right, right to left or in the middle.
 * Note that this container can be placed inside another container.
 * E.g. to have a button at the left and the right, do something like:
 * {@code HBox line = new HBox();}
 * <br>{@code HBox left = new HBox();}
 * <br>{@code left.add(Button);}
 * <br>{@code HBox right = new HBox(HBox.OPPOSITELINEAXIS);}
 * <br>{@code right.add(Button);}
 * <br>{@code line.add(left);}
 * <br>{@code line.add(right);}
 * <br>{@code form.add(line);}
 * @author Fred
 *
 */
public class HBox extends JComponent implements Scrollable {
	
	private static final long serialVersionUID = 1L;
	/**
	 * Used in constructor to lay out components centered on screen.
	 */
	public static final int CENTER = 1;
	/**
	 * Used in constructor to lay out components opposite the reading direction.
	 */
	public static final int OPPOSITELINEAXIS = 2;

	/**
	 * Constructor to layout components in the reading direction.
	 */
	public HBox() {
		
		this(false, false);
	}
	
	/**
	 * Constructor to layout components horizontally.
	 * @param orientation Can be CENTER or OPPOSITELINEAXIS.
	 */
	public HBox(int orientation) {
		
		super();
		switch(orientation) {
			case CENTER: super.setLayout(new HLayout(false, true)); break;
			case OPPOSITELINEAXIS: super.setLayout(new HLayout(true, false)); break;
			default: super.setLayout(new HLayout(false, false)); break;
		}
	}

	/**
	 * Constructor to layout components horizontally.
	 * <br> The use of HBox(int orientation) is preferred to this constructor though. 
	 * @param oppositeLineAxis Set to true to align components opposite the reading direction.
	 * @param centerLineAxis Set to true to center components/ place components in the middle.
	 */
	public HBox(boolean oppositeLineAxis, boolean centerLineAxis) {
		
		super();
		super.setLayout(new HLayout(oppositeLineAxis, centerLineAxis));
	}
	
	/**
	 * This overloaded method ensures that items aligned opposite the line axis
	 * remain orientated opposite the line axis.
	 */
	public void applyComponentOrientation(final ComponentOrientation co) {
		// Code overloads code in Component and Container.applyComponentOrientation
		
        if (co == null) {
            throw new NullPointerException();
        }
        if (co != ComponentOrientation.LEFT_TO_RIGHT && co != ComponentOrientation.RIGHT_TO_LEFT) {
        	super.applyComponentOrientation(co);
        	return;
        }
		final boolean isLeftToRight = getComponentOrientation().isLeftToRight();
		if (co.isLeftToRight() == isLeftToRight)
			return;
		setComponentOrientation(co);
		for (int i = 0; i < getComponentCount(); i++) {
			final Component c = getComponent(i);
			final boolean cltor = c.getComponentOrientation().isLeftToRight();
			if (cltor != co.isLeftToRight())
				c.applyComponentOrientation(co);
			// If component was already aligned opposite the line-axis ...
			if (cltor == co.isLeftToRight()) {
				// ... then align it opposite the line-axis again.
				if (co.isLeftToRight())
					c.applyComponentOrientation(ComponentOrientation.RIGHT_TO_LEFT);
				else
					c.applyComponentOrientation(ComponentOrientation.LEFT_TO_RIGHT);
			}
		}
	}
	
	/**
	 * To accomodate a scroller.
	 */
	public Dimension getPreferredScrollableViewportSize() {
		return this.getPreferredSize();
	}

	/**
	 * To accomodate a scroller.
	 */
	public int getScrollableUnitIncrement(final Rectangle arg0, final int arg1, final int arg2) {
		return (HVProps.defaultLineHeight - HVProps.smallHeightCorrection);
	}

	/**
	 * To accomodate a scroller.
	 */
	public int getScrollableBlockIncrement(final Rectangle arg0, final int arg1,
			final int arg2) {
		return (HVProps.defaultLineHeight - HVProps.smallHeightCorrection);
	}

	/**
	 * To accomodate a scroller.
	 */
	public boolean getScrollableTracksViewportWidth() {
		if (this.getParent().getWidth() > this.getMinimumSize().width)
			return true;
		return false;
	}

	/**
	 * To accomodate a scroller.
	 */
	public boolean getScrollableTracksViewportHeight() {
		if (this.getParent().getHeight() > this.getMinimumSize().height)
			return true;
		return false;
	}
	
	/**
	 * This (overloaded) convenience method allows adding components in the middle of
	 * a list of components (at the place indicated by index).
	 * @see java.awt.Container#add(java.awt.Component, int)
	 */
	public Component add(final Component comp, final int index) {
		 // The index is used as constraint and passed on to super().
		 // This makes it easier to insert a component in the HLayout
		 // or VLayout list of components.
		super.add(comp, Integer.valueOf(index));
		return comp;
	}
}
