/*
 * This software is licensed under the GNU Lesser GPL, see
 * http://www.gnu.org/licenses/ for details.
 * You can use this free software in commercial and non-commercial products,
 * as long as you give credit to this software when you use it.
 * This softwware is provided "as is" and the use of this software is 
 * completely at your own risk, there are absolutely no warranties.
 * 
 */

package net.java.dev.hvlayout;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.LayoutManager2;
import java.awt.Toolkit;
import java.util.ArrayList;

/**
 * A layout manager that aligns components horizontally. Components can be
 * aligned left to right, right to left and centered.
 * HLayout is a modified copy of VLayout. See VLayout for more information.
 * <br> VLayout and HLayout need to be kept in sync to work.  
 * @author Fred
 *
 */
public class HLayout implements LayoutManager2 {
	
	// See FormLayout for an explanation of this LineLayout
	// LineLayout is basically a copy of FormLayout but
	// works on width instead of height.
	// Also alignment like center or reverse (opposite line-axis)
	// is taken into account

	private transient ArrayList<Component> parts;
	private transient boolean reverse;
	private transient boolean center;
	private transient Dimension dmax = null;
	private transient Dimension dmin = null;
	private transient Dimension dpref = null;
	private transient int varMinWidth = 0;
	private transient double staticMaxWidth = 0;
	/**
	 * Size of screen, used in maximumLayoutSize()
	 */
	public static final Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	//public HVProps.FontDependentSizes previousDefaultSizes = null;
	
	/** Size of horizontal gap between two components in a line (horizontally outlined) */ 
	public transient int hGap;
	
	/**
	 * Creates a layout manager that aligns components in the reading direction.
	 * <br> HVProps.defaultHGap will be used as gap between the components.
	 *
	 */
	public HLayout() {
		
		this(false, false);
	}
	
	/**
	 * Creates a layout manager that aligns components in the reading direction.
	 * @param hGap The gap between the layed out components.
	 */
	public HLayout(int hGap) {
		
		this(false, false, hGap);
	}
	
	/**
	 * Creates a layout manager that aligns components horizontally.
	 * @param oppositeLineAxis Set to true to align components opposite the reading direction.
	 * @param centerLineAxis Set to true to center components/ place components in the middle.
	 */
	public HLayout(boolean oppositeLineAxis, boolean centerLineAxis) {
		this(oppositeLineAxis, centerLineAxis, HVProps.defaultHGap);
	}

	/**
	 * Creates a layout manager that aligns components horizontally.
	 * @param oppositeLineAxis Set to true to align components opposite the reading direction.
	 * @param centerLineAxis Set to true to center components/ place components in the middle.
	 * @param hGap The gap between the layed out components.
	 */
	public HLayout(boolean oppositeLineAxis, boolean centerLineAxis, int hGap) {
		
		super();
		parts = new ArrayList<Component>();
		reverse = oppositeLineAxis;
		center = centerLineAxis;
		this.hGap = hGap;
	}

	/** Adds a component at the end of the list or 
	 * if place is a valid index (Integer class required)
	 * adds the component at the specified
	 * index-place (0 is the first place). 
	 * <br> LineBox uses this method for a call to add(component, index)
	 * @param c The component to add
	 * @param place The place in the list of components for this layout. 
	 */
	public void addLayoutComponent(final Component c, final Object place) {

		if (place instanceof Integer) {
			final int index = ((Integer)place).intValue();
			if (index > -1 && (index < parts.size()))
				parts.add(index, c);
			else
				parts.add(c);
		} else {
			parts.add(c);
		}
	}
	
	/**
	 * Adds component c to the end of the list of components for this layout.
	 */
	public void addLayoutComponent(final String arg0, final Component c) {
		
		parts.add(c);
	}

	/**
	 * Removes component c from the list of components for this layout.
	 */
	public void removeLayoutComponent(final Component c) {

		boolean found = false;
		int i = 0;
		while (!found && i < parts.size()) {
			found = (parts.get(i) == c);
			i++;
		}
		if (found)
			parts.remove(--i);
	}

	public synchronized void invalidateLayout(final Container target) {

		dmax = null;
		dmin = null;
		dpref = null;
		//previousDefaultSizes = HVProps.getInstance().new FontDependentSizes(null);
	}

	/**
	 * Used by other layout managers to get an indication of the 
	 * relative place of this layout.
	 */
	public float getLayoutAlignmentX(final Container target) {

		final boolean leftToRight = target.getComponentOrientation().isLeftToRight();
		float f = 0.25f;
		if (leftToRight && reverse)
			f = 0.75f;
		if (!leftToRight && !reverse)
			f = 0.75f;
		if (center)
			f = 0.5f;
		return f;
	}

	/**
	 * Returns a reasonable value for this layout.
	 */
	public float getLayoutAlignmentY(final Container target) {

		return 0.25f;
	}

	/**
	 * The total gap width, used in resizing calculations.
	 * Note that this is a slightly different calculation then
	 * gapHeight() in VLayout. 
	 */
	private int gapWidth() {
		
		if (!parts.isEmpty())
			return (hGap * (parts.size() - 1));
		return 0;
	}
	
	/**
	 * The minimum size of this layout calculated by summing up
	 * all minimum sizes of the components in this layout.
	 */
	public Dimension minimumLayoutSize(final Container target) {

		if (dmin != null)
			return new Dimension(dmin.width, dmin.height);

		int width = 0;
		int maxHeight = 0;
		varMinWidth = 0;
		synchronized (target.getTreeLock()) {
			Dimension cmin;
			Dimension cpref;
			for(int i = 0; i < parts.size(); i++) {
				cmin = parts.get(i).getMinimumSize();
				cpref = parts.get(i).getPreferredSize();
				maxHeight = Math.max(maxHeight, cmin.height);
				width += cmin.width;
				if (cpref.width > cmin.width)
					varMinWidth += (cpref.width - cmin.width);
			}
		}
		final Dimension d = new Dimension(width + gapWidth(), maxHeight);
		dmin = new Dimension(d.width, d.height);
		return d;
	}

	/**
	 * The preferred size of this layout calculated by summing up
	 * all preferred sizes of the components in this layout.
	 */
	public Dimension preferredLayoutSize(final Container target) {
		
		if (dpref != null)
			return new Dimension(dpref.width, dpref.height);

		int width = 0;
		int maxHeight = 0;
		staticMaxWidth = 0;
		synchronized (target.getTreeLock()) {
			Dimension cpref;
			Dimension cmax;
			for(int i = 0; i < parts.size(); i++) {
				cpref = parts.get(i).getPreferredSize();
				cmax = parts.get(i).getMaximumSize();
				maxHeight = Math.max(maxHeight, cpref.height);
				width += cpref.width;			
				if (cpref.width >= cmax.width)
					staticMaxWidth += cpref.width;
			}
		}
		final Dimension d = new Dimension(width + gapWidth(), maxHeight);
		staticMaxWidth += gapWidth();
		dpref = new Dimension(d.width, d.height);
		return d;
	}

	/**
	 * The maximum size of this layout calculated by summing up
	 * all maximum sizes of the components in this layout.
	 * <br> Note that for resizing calculations in layoutContainer(), 
	 * the maximum width is always set to screenSize.width.
	 */
	public Dimension maximumLayoutSize(final Container target) {

		if (dmax != null)
			return new Dimension(dmax.width, dmax.height);
		/*
		 * Following piece makes JList in a scroller behave better in an HBox.
		 */

		int maxHeight = 0;
//		int width = 0;
		synchronized (target.getTreeLock()) {
			Dimension cmax;
			for(int i = 0; i < parts.size(); i++) {
				cmax = parts.get(i).getMaximumSize();
//				if (cmax.width > 0)
//					width += hGap + cmax.width;
				maxHeight = Math.max(maxHeight, cmax.height);
			}
		}
		// Set max width always to (fixed) max possible size so that mixed line-boxes like
		// left 	middle 		right
		// always have the correct space between them.
		final Dimension d = new Dimension(screenSize.width, maxHeight);
		dmax = new Dimension(d.width, d.height);
		return d;
	}

	/**
	 * Lays out the components as specified by their sizes.
	 * <br> Always tries to make the most use of the given area.
	 * <br> Some small calculations are done for vertical size to 
	 * allow components to use all available vertical room when it is available.
	 */
	public void layoutContainer(final Container target) {

		if (center)
			reverse = false;
		boolean ltor = target.getComponentOrientation().isLeftToRight();
		if (reverse)
			ltor ^= true; // Faster then ltor = !ltor
		final Insets targetInsets = target.getInsets();
		int maxWidth = target.getWidth() - targetInsets.left - targetInsets.right;
		boolean smallest;
		boolean shrink;
		boolean grow;
		
		synchronized (target.getTreeLock()) {
			final Dimension dminimum = minimumLayoutSize(target); 
			final Dimension dpreferred = preferredLayoutSize(target);
			maxWidth = Math.max(maxWidth, dminimum.width);
			final int maxHeight = target.getHeight() - targetInsets.top - targetInsets.bottom;
			smallest = maxWidth <= dminimum.width;
			shrink = !smallest && (maxWidth < dpreferred.width) && (varMinWidth > 0);
			grow = !smallest && !shrink && (maxWidth > dpreferred.width) && (dpreferred.width  > staticMaxWidth);
			double widthFactor = 0;
			if (shrink)
				widthFactor = (double)(maxWidth - dminimum.width) / varMinWidth;
			if (grow)
				widthFactor = (double)(maxWidth - dpreferred.width) / (dpreferred.width - staticMaxWidth);

//System.out.println("maxW: " + maxWidth + " shrink: " + shrink + " wFactor: " + widthFactor);			
			
			double lineWidth = (ltor ? targetInsets.left : maxWidth + targetInsets.left);
			if (center && (maxWidth >= dpreferred.width)) {
				if (ltor)
					lineWidth = (maxWidth / 2) - (dpreferred.width / 2);
				else
					lineWidth = (maxWidth / 2) + (dpreferred.width / 2);
			}
			Dimension cmin;
			Dimension cpref;
			Component c;
			for (int i = 0; i < parts.size(); i++) {
				if (reverse)
					c = parts.get(parts.size() - i - 1);
				else
					c = parts.get(i);
				cmin = c.getMinimumSize();
				cpref = c.getPreferredSize();
				int cheight = (maxHeight > cmin.height ? maxHeight : cmin.height);
				if (maxHeight > cpref.height)
					cheight = cpref.height;
				if ((cpref.height > cmin.height) && (cpref.height <= maxHeight))
					cheight = maxHeight; // Not fixed in vertical size, give all available room to expand vertical
				double width = (smallest ? cmin.width : cpref.width);
				boolean hasStaticWidth = true;
				if (!smallest) {
					if (shrink && cmin.width < cpref.width)
						hasStaticWidth = false;
					if (grow && cpref.width < c.getMaximumSize().width)
						hasStaticWidth = false;
				}
				if (shrink && !hasStaticWidth) {
					width = (int)(cmin.width + ((cpref.width - cmin.width) * widthFactor)); 
				}
				if (grow && !reverse && !center && !hasStaticWidth) {
					width = (int)(cpref.width + (cpref.width * widthFactor));
				}
				if (ltor) {
					c.setBounds((int)lineWidth, targetInsets.top, 
							(int)width, cheight);
					lineWidth += width + hGap;
				}
				if (!ltor) {
					c.setBounds((int)lineWidth - (int)width, targetInsets.top, 
							(int)width, cheight);
					lineWidth -= width + hGap;
				}
			}
		}
	}
}
