/*
 * This software is licensed under the GNU Lesser GPL, see
 * http://www.gnu.org/licenses/ for details.
 * You can use this free software in commercial and non-commercial products,
 * as long as you give credit to this software when you use it.
 * This softwware is provided "as is" and the use of this software is 
 * completely at your own risk, there are absolutely no warranties.
 * 
 */
package net.java.dev.hvlayout;

import java.awt.Image;
import java.awt.Insets;
import java.awt.event.MouseEvent;

import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JButton;

/**
 * A button that shows a tooltip when text is truncated
 * and no tooltip-text is set. Also supports a button
 * with only an icon set.
 * @author Fred
 *
 */
public class HVButton extends JButton {
	
	private static final long serialVersionUID = -706109726778958796L;

	/** 
	 * The margins are the space between the button-border and the text
	 * and/or icon. Setting to 0 has no effect on the text but does
	 * allow the icon on the button to fully display (else icon
	 * is made smaller by the margins).
	 * The buttonMargins are applied to all instances of this class.
	 */
	public static Insets buttonMargins = new Insets(0, 0, 0, 0);

	public HVButton() {
        this(null, null);
    }

    public HVButton(Action a) {
        this();
        setAction(a);
    }

    /** 
     * Returns a square button showing the icon. 
     * Assumes this button ONLY shows an icon. 
     * If icon is an instance of ImageIcon,
     * the icon is scaled to fit the square button
     * and the tooltip is set to the icon-description.
     */
    public HVButton(Icon icon) {
        super();
		HVProps.setSize(this, HVProps.defaultLineHeight, HVProps.defaultLineHeight, true);
		setMargin(buttonMargins);
        setIconTextGap(0);
        setIcon(icon);
		setRequestFocusEnabled(true);
    }

    public HVButton(String text) {
        this(text, null);
    }

    public HVButton(String text, Icon icon) {
    	super(text);
		HVProps.setSize(this, 0, 0, true);
		setMargin(buttonMargins);
		setIconTextGap(2);
		setToolTipText(" ");
		setIcon(icon);
		setRequestFocusEnabled(true);
    }
    
	/** Same as HVLabel.getToolTipText() */
    public String getToolTipText(MouseEvent e) {

		String text = super.getToolTipText();
		if (!text.equals(" ")) {
			return text;
		}
		text = getText();
		int textWidth = getFontMetrics(getFont()).stringWidth(text);
		int iconWidth = (getIcon() == null ? 0 : getIcon().getIconWidth() + getIconTextGap());
		int textSpace = getWidth() - iconWidth 
			- getInsets().left - getInsets().right;
		return (textWidth > textSpace ? text : null);
	}
    
   /** 
    * If icon is an instance of ImageIcon,
    * the icon is scaled to fit the largest square on the button
    * and the tooltip is set to the icon-description.
    */
	public void setIcon(Icon icon) {

		if (icon != null && icon instanceof ImageIcon) {
			// Tried using borderSize here but that has the same effect
			// as calling getInsets.
			int h = getPreferredSize().height - getInsets().top	- getInsets().bottom;
			ImageIcon sizedIcon = new ImageIcon(((ImageIcon)icon).getImage().getScaledInstance(h, h, Image.SCALE_SMOOTH));
			super.setIcon(sizedIcon);
			if (getText() == null || getText().length() == 0 || getText().equals(" ")) {
				setToolTipText(((ImageIcon)icon).getDescription());
			}
		} else {
			super.setIcon(icon);
		}
	}
}
