/*
 * This software is licensed under the GNU Lesser GPL, see
 * http://www.gnu.org/licenses/ for details.
 * You can use this free software in commercial and non-commercial products,
 * as long as you give credit to this software when you use it.
 * This softwware is provided "as is" and the use of this software is 
 * completely at your own risk, there are absolutely no warranties.
 * 
 */

package net.java.dev.hvlayout;

import java.awt.Component;
import java.awt.ComponentOrientation;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.Rectangle;

import javax.swing.JComponent;
import javax.swing.Scrollable;
import javax.swing.border.EmptyBorder;

/**
 * A container that lays out components vertically (top to bottom). This container
 * can be placed inside a scroller.
 */
public class VBox extends JComponent implements Scrollable {
	
	private static final long serialVersionUID = 1L;

	/**
	 * Constructs a container that that lays out components from top to bottom.
	 * Uses no border and HVProps.defaultVGap for spacing components.
	 */
	public VBox() {
		
		super();
		super.setLayout(new VLayout());
	}
	
	/**
	 * Constructs a container that that lays out components from top to bottom
	 * and puts a vGap vertical space between the components
	 * and has no border.
	 * @param vGap The vertical space between components in this container. 
	 * Use VBox(vGap, null) for the top-container to set a border if needed.
	 */
	public VBox(final int vGap) {
		
		this(vGap, HVProps.noInsets);
	}
	/**
	 * Constructs a container that that lays out components from top to bottom.
	 * Puts a vGap vertical space between the components
	 * Setting border insets can help to make the container appear as a 
	 * different section on the screen.
	 * @param borderInsets The insets for this container. Null sets
	 * an empty border with HVProps.defaultBorder.
	 */
	public VBox(final Insets borderInsets) {
		
		this(HVProps.defaultVGap, borderInsets);
	}

	/** 
	 * Constructs a container that that lays out components from top to bottom.
	 * Uses vGap to put space between components and borderInsets as empty border.
	 * @param vGap See VBox(int vGap)
	 * @param borderInsets See VBox(Insets borderInsets)
	 */
	public VBox(final int vGap, final Insets borderInsets) {
		
		super();
		super.setLayout(new VLayout(vGap));
		if (borderInsets == null) {
			setBorder(HVProps.defaultBorder);
		} else if (borderInsets != HVProps.noInsets) {
			setBorder(new EmptyBorder(borderInsets));
		}
	}

	/**
	 * This overloaded method ensures that items aligned opposite the line axis
	 * remain orientated opposite the line axis.
	 */
	public void applyComponentOrientation(final ComponentOrientation co) {
		// Code overwrites code in Component and Container.applyComponentOrientation
		
        if (co == null) {
            throw new NullPointerException();
        }
        if (co != ComponentOrientation.LEFT_TO_RIGHT && co != ComponentOrientation.RIGHT_TO_LEFT) {
        	super.applyComponentOrientation(co);
        	return;
        }
		final boolean isLeftToRight = getComponentOrientation().isLeftToRight();
		if (co.isLeftToRight() == isLeftToRight)
			return;
		setComponentOrientation(co);
		for (int i = 0; i < getComponentCount(); i++) {
			final Component c = getComponent(i);
			final boolean cltor = c.getComponentOrientation().isLeftToRight();
			// Check if component is not orientated opposite the line axis
			if (cltor != co.isLeftToRight())
				c.applyComponentOrientation(co);
			if (cltor == co.isLeftToRight()) {
				// Component is orientated opposite the line axis
				if (co.isLeftToRight())
					c.applyComponentOrientation(ComponentOrientation.RIGHT_TO_LEFT);
				else
					c.applyComponentOrientation(ComponentOrientation.LEFT_TO_RIGHT);
			}
		}
	}

	/** To accomodate a scroller */
	public Dimension getPreferredScrollableViewportSize() {
		return this.getPreferredSize();
	}

	/** To accomodate a scroller */
	public int getScrollableUnitIncrement(final Rectangle arg0, final int arg1, 
			final int arg2) {
		return (HVProps.defaultLineHeight - HVProps.smallHeightCorrection);
	}

	/** To accomodate a scroller */
	public int getScrollableBlockIncrement(final Rectangle arg0, final int arg1,
			final int arg2) {
		return(HVProps.defaultLineHeight - HVProps.smallHeightCorrection);
	}

	/** To accomodate a scroller */
	public boolean getScrollableTracksViewportWidth() {
		if (this.getParent().getWidth() > this.getMinimumSize().width)
			return true;
		return false;
	}

	/** To accomodate a scroller */
	public boolean getScrollableTracksViewportHeight() {
		if (this.getParent().getHeight() > this.getMinimumSize().height)
			return true;
		return false;
	}

	/**
	 * This (overloaded) convenience method allows adding components in the middle of
	 * a list of components (at the place indicated by index).
	 * @see java.awt.Container#add(java.awt.Component, int)
	 */
	public Component add(final Component comp, final int index) {
		super.add(comp, Integer.valueOf(index));
		return comp;
	}
}
