package net.java.dev.fwutil.pf;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
//import java.awt.event.ActionEvent;
//import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;

import net.java.dev.fwutil.Log;
import net.java.dev.fwutil.StringListMap;
import net.java.dev.fwutil.Util;

public class DiskInfo {
	
	public static final String RootDir = Util.emptyString;
	public static final String BookMarks = "BookMarks";
	
	public static boolean windowsOS = Util.isWindowsOS();
	
	private static StringListMap<FileInfo> files = new StringListMap<FileInfo>();
	static {
		// TODO: Make map synchronized?
		//files = (StringListMap<FileInfo>) Collections.synchronizedMap(files);
		files.uniqueValues = true;
		if (windowsOS) {
			File[] rootDirectories = File.listRoots();
			for (File f : rootDirectories) files.add(RootDir, new FileInfo(f.getAbsolutePath(), true));
			files.caseSensitive = false;
			Log.log("Added " + rootDirectories.length + " entries to root.");
		} else {
			File rootDirectory = new File("/");
			File[] rootFiles = rootDirectory.listFiles();
			for (File f : rootFiles) files.add(RootDir, new FileInfo(f));
			Log.log("Added " + rootFiles.length + " entries to root.");
		}
		Log.log("File names are case-" + (files.caseSensitive ? "sensitive" : "insensitive"));
	}
	
	public DiskInfo() {
		super();
	}
	
	public static boolean isCaseSensitive() {
		return files.caseSensitive;
	}

	public List<FileInfo> getRootDirectories(final boolean directoriesOnly) {
		return getDirectoryContents(RootDir, directoriesOnly, true);
	}
	
	public List<FileInfo> getDirectoryContents(final String pathKey, 
			final boolean directoriesOnly, final boolean showHidden) {

		if (!files.containsKey(pathKey))
			refreshDirectoryContents(pathKey);
		List<FileInfo> dirContents = files.clone(files.get(pathKey));
		if (dirContents == null) return new ArrayList<FileInfo>();
		if (directoriesOnly || !showHidden ) {
			int i = 0;
			while (i < dirContents.size()) {
				if (directoriesOnly && !dirContents.get(i).isDirectory) {
					dirContents.remove(i);
				} else if (!showHidden && dirContents.get(i).isHidden) {
					dirContents.remove(i);
				} else {
					i++;
				}
			}
		}
		Log.log("Returning " + dirContents.size() + " entries for " + pathKey);
		return dirContents;
	}
	
	public void refreshDirectoryContents(final String pathKey) {
		
		File inFile = new File(pathKey);
		if (inFile.exists()) {
			if (inFile.isDirectory()) {
				//busyListener.actionPerformed(new ActionEvent(this, 0, "loadingFileList"));
				Log.log("Looking up files for " + pathKey);
				File[] dirFiles = inFile.listFiles();
				files.remove(pathKey);
				for (File f : dirFiles) {
					files.add(pathKey, new FileInfo(f));
				}
				Log.log("Looked up " + dirFiles.length + " files for " + pathKey);
				//busyListener.actionPerformed(new ActionEvent(this, 0, "doneLoadingFileList"));
			} else {
				Log.log("No directory contents to refresh for file " + inFile.getName());
			}
		} else {
			Log.logWarning("Path is not a valid file or directory: " + pathKey);
		}
	}

	public static FileInfo getUserHome() {
	
		FileInfo userHome = new FileInfo();
		userHome.isDirectory = true;
		userHome.name = (windowsOS ? Util.userFileDir : Util.userHome);
		File h = new File(userHome.name);
		userHome.modificationDate = h.lastModified();
		return userHome;
	}

	public static String getPrefixName(final List<FileInfo> fl) {
		
		int prefixSize = fl.get(0).name.length();
		Iterator<FileInfo> it = fl.iterator();
		while (prefixSize > 0 && it.hasNext()) {
			FileInfo fi = it.next();
			prefixSize = Math.min(prefixSize, fi.name.length());
			boolean common = true;
			int i = 0;
			while (common && i < prefixSize) {
				if (windowsOS) {
					common = String.valueOf(fl.get(0).name.charAt(i)).toUpperCase()
						.equals(String.valueOf(fi.name.charAt(i)).toUpperCase());
				} else {
					common = fl.get(0).name.charAt(i) == fi.name.charAt(i);
				}
				if (common) i++;
			}
			prefixSize = i;
		}
		return (prefixSize == 0 ? null : fl.get(0).name.substring(0, prefixSize));
	}

	public static List<FileInfo> filterFileList(final List<FileInfo> fl, String prefix) {
		
		if (prefix == null || prefix.length() == 0 || fl.size() < 2) return fl;
		ArrayList<FileInfo> filteredList = new ArrayList<FileInfo>();
		if (windowsOS) prefix = prefix.toUpperCase();
		boolean search = prefix.startsWith("*");
		if (search && prefix.length() == 1) return fl;
		String[] searchTokens = null;
		if (search) searchTokens = prefix.substring(1).split(" ");
		Log.log("Filtering files on: " + prefix);
		for (FileInfo fi : fl) {
			if (search) {
				boolean containsTokens = true;
				int i = 0;
				while (i < searchTokens.length && containsTokens) {
					if (windowsOS) {
						containsTokens = fi.name.toUpperCase().contains(searchTokens[i]);
					} else {
						containsTokens = fi.name.contains(searchTokens[i]);
					}
					i++;
				}
				if (containsTokens) {
					filteredList.add(fi);
				}
			} else {
				if (windowsOS) {
					if (fi.name.toUpperCase().startsWith(prefix)) filteredList.add(fi);
				} else {
					if (fi.name.startsWith(prefix)) filteredList.add(fi);
				}
			}
		}
		return (filteredList.size() > 0 ? filteredList : fl);
	}

	public static String getPathKey(final File existingDir) {
		
		String dirPathKey = existingDir.getAbsolutePath();
		try {
			dirPathKey = existingDir.getCanonicalPath();
		} catch (IOException ieo) {
			Log.logError("Problem getting canonical path for " + dirPathKey + "\n" + Log.exception2String(ieo));
		}
		dirPathKey = endWithSeparator(dirPathKey);
		return dirPathKey;
	}

	public static String endWithSeparator(final String path) {
		
		if (path.endsWith(File.separator)) return path;
		return path + File.separator;
	}
}
