package net.java.dev.fwutil.pf.ui.table;

import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.io.File;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.JComponent;
import javax.swing.KeyStroke;
import javax.swing.SortOrder;
import javax.swing.RowSorter.SortKey;
import javax.swing.table.TableRowSorter;

import net.java.dev.fwutil.Log;
import net.java.dev.fwutil.MTable;
import net.java.dev.fwutil.pf.FileInfo;
import net.java.dev.fwutil.pf.ui.BrowseLocation;

public class FileTable extends MTable {
	
	private static final long serialVersionUID = -3290950480549155332L;
	
	public static final String SortKeyId = "-sortkeys";

	private FileTableDataModel fileDataModel;
	private BrowseLocation browseLocation;

	@SuppressWarnings("serial")
	public FileTable(final BrowseLocation browseLocation, final String tableId) {
		super(tableId);
		this.browseLocation = browseLocation;
		fileDataModel = new FileTableDataModel();
		setModel(fileDataModel);
		TableRowSorter<FileTableDataModel> rowSorter = new TableRowSorter<FileTableDataModel>(fileDataModel);
		// Ignore case when sorting file-names.
		rowSorter.setComparator(FileTableColumns.Name.ordinal(), 
				new Comparator<String>() {
					public int compare(String o1, String o2) {
						return (o1 == o2 ? 0 : o1.compareToIgnoreCase(o2));
					}
				});
		rowSorter.setComparator(FileTableColumns.Extension.ordinal(), 
				new Comparator<String>() {
					public int compare(String o1, String o2) {
						return (o1 == o2 ? 0 : o1.compareToIgnoreCase(o2));
					}
				});
		setRowSorter(rowSorter);

		getColumnModel().getColumn(FileTableColumns.Type.ordinal()).setCellRenderer(new FileTypeCellrenderer());  
		getColumnModel().getColumn(FileTableColumns.Size.ordinal()).setCellRenderer(new FileSizeCellRenderer());  
		
		getInputMap(JComponent.WHEN_FOCUSED).put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), "openTableRow");
		getActionMap().put("openTableRow", new AbstractAction() {
			public void actionPerformed(final ActionEvent ae) {
				browseLocation.openFileInSelectedTableRow();
			}});

		applyUIProps();
	}
	
	/**
	 * Overloaded to set default column width etc.
	 */
	protected void setDefaultUIProps() {

		FileTableColumns[] fc =  FileTableColumns.values();
		int max  = Math.min(getColumnCount(), fc.length);
		for (int i = 0; i < max; i++) {
			getColumnModel().getColumn(i).setPreferredWidth(fc[i].defaultWidth);
		}
		ArrayList<SortKey> skl = new ArrayList<SortKey>();
		skl.add(new SortKey(FileTableColumns.Type.ordinal(), SortOrder.DESCENDING));
		skl.add(new SortKey(FileTableColumns.Name.ordinal(), SortOrder.ASCENDING));
		getRowSorter().setSortKeys(skl);
	}
	
	public FileTableDataModel getModel() {
		return fileDataModel;
	}
	
	public FileInfo getFileInfo(final int selectedRowView) {
		return fileDataModel.getRow(getRowSorter().convertRowIndexToModel(selectedRowView));
	}
	
	/** Updates the row in the table in the UI with the given FileInfo. */
	public void updateFileUI(FileInfo fi) {
		
		List<FileInfo> fileList = fileDataModel.activeFileList;
		if (fileList != null) {
			boolean found = false;
			int i = 0;
			while (!found && i < fileList.size()) {
				found = fileList.get(i) == fi;
				i++;
			}
			if (found) fileDataModel.fireTableRowsUpdated(i-1, i-1);
			else Log.log("Could not find row to update for " + fi.name);
		}
	}
	
	public void setSelectedRow(final int rowIndex) {
		
		if (rowIndex > -1 && rowIndex < fileDataModel.getRowCount()) {
			final int viewRow = getRowSorter().convertRowIndexToView(rowIndex);
			setRowSelectionInterval(viewRow, viewRow);
		}
	}
	
	public File getSelectedFile() {
		
		File f = null;
		if (getSelectedRow() > -1) {
			String filePath = browseLocation.getLastPathKey() + getFileInfo(getSelectedRow()).name;
			f = new File(filePath);
		}
		return f;
	}
	
	public File getFile(final int row) {
		
		String filePath = browseLocation.getLastPathKey() + getFileInfo(row).name;
		File f = new File(filePath);
		return f;
	}
	
	public String getLastPathKey() {
		return browseLocation.getLastPathKey();
	}
	
	public String getFilePath(final int row) {
		return browseLocation.getLastPathKey() + getFileInfo(row).name;
	}
	
	public void sortOnName() {
		
		if (toggleSortOrder(FileTableColumns.Type.ordinal(), FileTableColumns.Name.ordinal()))
			return;
		ArrayList<SortKey> skl = new ArrayList<SortKey>();
		skl.add(new SortKey(FileTableColumns.Type.ordinal(), SortOrder.DESCENDING));
		skl.add(new SortKey(FileTableColumns.Name.ordinal(), SortOrder.ASCENDING));
		getRowSorter().setSortKeys(skl);
	}
	
	public void sortOnModified() {

		if (toggleSortOrder(-1, FileTableColumns.Modified.ordinal()))
			return;
		ArrayList<SortKey> skl = new ArrayList<SortKey>();
		skl.add(new SortKey(FileTableColumns.Modified.ordinal(), SortOrder.DESCENDING));
		getRowSorter().setSortKeys(skl);
	}
	
	public void sortOnSize() {
		
		if (toggleSortOrder(-1, FileTableColumns.Size.ordinal()))
			return;
		ArrayList<SortKey> skl = new ArrayList<SortKey>();
		skl.add(new SortKey(FileTableColumns.Size.ordinal(), SortOrder.DESCENDING));
		getRowSorter().setSortKeys(skl);
	}
}
