package nl.intercommit.mario.tools;

import java.beans.XMLEncoder;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.Serializable;
import java.io.UnsupportedEncodingException;
import java.net.InetAddress;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Properties;
import java.util.Set;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageProducer;
import javax.jms.ObjectMessage;
import javax.jms.Queue;
import javax.jms.QueueBrowser;
import javax.jms.QueueConnectionFactory;
import javax.jms.Session;
import javax.jms.TextMessage;
//import javax.management.Attribute;
import javax.management.MBeanAttributeInfo;
import javax.management.MBeanInfo;
import javax.management.MBeanServerConnection;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;
import javax.naming.InitialContext;
import javax.naming.NameClassPair;
import javax.naming.NamingException;

import org.apache.log4j.Logger;
import org.jboss.jms.server.messagecounter.MessageStatistics;
import org.jboss.security.client.SecurityClient;
import org.jboss.security.client.SecurityClientFactory;

public final class JmsUtil {

	private static Logger log = Logger.getLogger(JmsUtil.class.getName());
	private static final JmsUtil thisInstance = new JmsUtil();

	/** See the Jboss-Jms user guide. */
	public static final String[] queueAtts = new String[] {"Name", "JNDIName", "DLQ", "ExpiryQueue",
		"RedeliveryDelay", "MaxDeliveryAttempts", "FullSize", "PageSize",
		"CreatedProgrammatically", "MessageCount", "ScheduledMessageCount",
		"MaxSize", "Clustered", "MessageStatistics",
		"MessageCounterHistoryDayLimit", "ConsumerCount"};
	// Missing in list because it gives a Serializable error when retrieved: "MessageCounter"

	public static InitialContext getContext() {

		InitialContext ic = null;
		try {
			ic = new InitialContext(ConfUtil.loadJNDI());
		} catch (NamingException e) {
			log.error("Could not retrieve ConnectionFactory: ", e);
		}
		return ic;
	}

	/** Close context if it exists. */
	public static void closeContext(final InitialContext ic) {
		try {
			if (ic != null) ic.close();
		} catch (NamingException e) {
			log.error("Could not close connection " + ic +": ", e);
		}
	}

	public static ConnectionFactory getConnectionFactory(final InitialContext ic) {

		ConnectionFactory cf = null;
		try {
			cf = (ConnectionFactory)ic.lookup("/ConnectionFactory");
		} catch (NamingException e) {
			log.error("Could not retrieve ConnectionFactory: ", e);
		}
		return cf;
	}

	/**
	 * Closes connection if it exists. This will also close any sessions
	 * associated with the connection.
	 */
	public static void closeConnection(final Connection con) {
		try {
			if (con != null) con.close();
		} catch(JMSException jmse) {
			log.error("Could not close connection " + con +": ", jmse);
		}
	}

	public static MBeanServerConnection getMBeanServer(final InitialContext ic) {

		MBeanServerConnection mserver = null;
		try {
			mserver = (MBeanServerConnection) ic.lookup("jmx/invoker/RMIAdaptor");
		} catch (NamingException e) {
			log.error("Could not retrieve MBeanServer: ", e);
		}
		return mserver;
	}

	public static boolean JBossRunning(final MBeanServerConnection msc) {

		Boolean started = Boolean.FALSE;
		try {
			ObjectName jserver = new ObjectName("jboss.system:type=Server");
			started = (Boolean)msc.getAttribute(jserver, "Started");
		} catch (Exception e) {
			log.error("Failed to get JBoss server status:", e);
		}
		return started.booleanValue();
	}

	public static QueueConnectionFactory getQueueConnectionFactory(final InitialContext ic) {

		QueueConnectionFactory qcf = null;
		try {
			qcf = (QueueConnectionFactory) ic.lookup("QueueConnectionFactory");
		} catch (NamingException e) {
			log.error("Could not retrieve queue connection factory: ", e);
		}
		return qcf;
	}

	public static String[] getDomainNames(final MBeanServerConnection mserver) {

		String[] doms = null;
		try {
			doms = mserver.getDomains();
		} catch (IOException e) {
			log.error("Could not retrieve domains: ", e);
		}
		return doms;
	}

	public static List<String> getQueueNames(final InitialContext ic) {

		if (ic == null)
			return null;
		LinkedList<String> l = new LinkedList<String>();
		try {
			Enumeration<NameClassPair> qnames = ic.list("queue");
			while (qnames.hasMoreElements())
				l.add(qnames.nextElement().getName());
		} catch (NamingException e) {
			log.error("Could not retrieve queue names: ", e);
			l = null;
		}
		return l;
	}

	/** Returns true if the queueName exists. */
	public static boolean queueExists(final InitialContext ic, final String queueName) {

		boolean qexists = false;
		try {
			@SuppressWarnings("unused")
			Queue q = (Queue)ic.lookup("queue/" + queueName);
			qexists = true;
		} catch (NamingException unused) {}
		return qexists;
	}

	public static HashMap<String, Object> getQueueAttValues(final MBeanServerConnection mserver,
			final String queueName, final String[] queueAttributes) {

		HashMap<String, Object> nameValue = new HashMap<String, Object>();
		try {
			ObjectName qn = new ObjectName("jboss.messaging.destination:service=Queue,name=" + queueName);
			for (int i = 0; i < queueAttributes.length; i++) {
				nameValue.put(queueAttributes[i], mserver.getAttribute(qn, queueAttributes[i]));
			}
		} catch (Exception e) {
			log.error("Could not retrieve queue attributes for queue " + queueName + ": ", e);
			nameValue = null;
		}
		return nameValue;
	}

	public static HashMap<String, Object> getQueueAttValues(final MBeanServerConnection mserver, final String queueName) {

		return getQueueAttValues(mserver, queueName, queueAtts);
	}
	/**
	 * Returns true when message statistics are gathered on the JBoss server.
	 * See also enableQueueStats.
	 * Requires ConfUtil.setSecurity(null) to be executed first, else will
	 * throw a "not allowed, no security" error.
	 */
	@SuppressWarnings("unchecked")
	public static boolean hasQueueStats(final MBeanServerConnection mserver) {

		boolean hasStats = false;
		try {
			ObjectName spon = new ObjectName("jboss.messaging:service=ServerPeer");
			List<MessageStatistics> mstats = (List<MessageStatistics>) mserver.getAttribute(spon, "MessageStatistics");
			hasStats = (mstats != null && mstats.size() > 0 &&
					mstats.get(0).getTimeLastUpdate() > System.currentTimeMillis() - 30000);
			mstats.clear();
		} catch (Exception e) {
			log.error("Could not determine if message queue statistics are on.", e);
		}
		return hasStats;
	}

	/**
	 * Configures JBoss to gather queue message statistics or not.
	 * See also http://www.jboss.org/file-access/default/members/jbossmessaging/freezone/docs/javadoc/api-1.4.0.SP1/org/jboss/jms/server/ServerPeer.html
	 * @param mserver
	 * @param on If true, message statistics are set as specified, else false (also on error).
	 */
	public static boolean enableQueueStats(final MBeanServerConnection mserver, final boolean on) {

		boolean ok = false;
		try {
			ObjectName spon = new ObjectName("jboss.messaging:service=ServerPeer");
			if (on) {
				mserver.invoke(spon, "enableMessageCounters", null, null);
				// Does not work, get "Unable to find operation" error.
				/*				mserver.invoke(spon, "setDefaultMessageCounterHistoryDayLimit",
						new Object[] {(Integer)1},
						new String[]{"java.lang.Integer"});
				 */				ok = true;
			} else {
				mserver.invoke(spon, "disableMessageCounters", null, null);
				ok = true;
			}
		} catch (Exception e) {
			log.error("Could not put queue stats gathering to " + on + ".", e);
			e.printStackTrace();
			ok = false;
		}
		return ok;
	}


	/**
	 * Returns the frequency at which JBoss refreshed the queue message statistics.
	 * Default is 10000 which is what this method returns on error.
	 */
	public static long getQueueStatsUpdateDelay(final MBeanServerConnection mserver) {

		long delay = 10000;
		try {
			ObjectName spon = new ObjectName("jboss.messaging:service=ServerPeer");
			delay = (Long) mserver.getAttribute(spon, "MessageCounterSamplePeriod");
		} catch (Exception e) {
			log.error("Could not get queue stats update delay.", e);
		}
		return delay;
	}

	/**
	 * Gets the message counter statistics RMI objects for all queues.
	 * Requires ConfUtil.setSecurity(null) to be executed first, else will
	 * throw a "not allowed, no security" error.
	 * Also requires message statistics to be on, see enableQueueStats
	 * @param mserver
	 */
	@SuppressWarnings("unchecked")
	public static List<MessageStatistics> getQueueCounterStats(final MBeanServerConnection mserver) {

		List<MessageStatistics> mstats = null;
		try {
			ObjectName spon = new ObjectName("jboss.messaging:service=ServerPeer");
			mstats = (List<MessageStatistics>) mserver.getAttribute(spon, "MessageStatistics");
		} catch (Exception e) {
			log.error("Could not get message counter statistics.", e);
		}
		return mstats;
	}


	/** Returns the number of messages on the queue. */
	public static int getQueueSize(final MBeanServerConnection mserver, final String queueName) {

		Integer qs = null;
		try {
			ObjectName qn = new ObjectName("jboss.messaging.destination:service=Queue,name=" + queueName);
			qs = (Integer)mserver.getAttribute(qn, "MessageCount");
		} catch (Exception e) {
			log.error("Could not retrieve queue size for queue " + queueName + ": ", e);
		}
		if (qs == null)
			return -1;
		return qs.intValue();
	}

	/** Resets all message counters. */
	public static boolean resetQueueStats(final MBeanServerConnection mserver) {

		ObjectName spon;
		boolean ok = true;
		try {
			spon = new ObjectName("jboss.messaging:service=ServerPeer");
			mserver.invoke(spon, "resetAllMessageCounters", null, null);
		} catch (Exception e) {
			ok = false;
			log.error("Could not reset message counters.", e);
		}
		return ok;
	}

	/** Resets message counter for queue. */
	public static boolean resetQueueStats(final MBeanServerConnection mserver, final String queueName) {

		ObjectName qon;
		boolean ok = true;
		try {
			qon = new ObjectName("jboss.messaging.destination:service=Queue,name=" + queueName);
			mserver.invoke(qon, "resetMessageCounter", null, null);
		} catch (Exception e) {
			ok = false;
			log.error("Could not reset message counters.", e);
		}
		return ok;
	}

	public static List<Message> getMessages(final InitialContext ic, final String queueName,
			final String userName, final String passWord) {

		Connection con = null;
		LinkedList<Message> msgs = new LinkedList<Message>();
		try {
			Queue queue = (Queue)ic.lookup("queue/" + queueName);
			ConnectionFactory cf = getConnectionFactory(ic);
			if (userName == null || userName.length() == 0) {
				con = cf.createConnection();
			} else {
				con = cf.createConnection(userName, passWord);
			}
			Session session = con.createSession(false, Session.AUTO_ACKNOWLEDGE);

			QueueBrowser qbrowser = session.createBrowser(queue);
			Enumeration<?> qamsgs = qbrowser.getEnumeration();
			while (qamsgs.hasMoreElements())
				msgs.add((Message)qamsgs.nextElement());
			qbrowser.close();
		} catch (Exception e) {
			log.error("Could not get messages from queue " + queueName + ": ", e);
			msgs = null;
		} finally {
			closeConnection(con);
		}
		return msgs;
	}

	public static void printMessageList(final List<Message> msgs) {

		if (msgs == null) {
			log.error("Message list does not exist.");
			return;
		}
		if (msgs.size() == 0)
			log.warn("Message list is empty.");
		try {
			for (Message jmsg : msgs) {
				System.out.println(jmsg.getJMSMessageID() + ": " + jmsg.getJMSType() + " - " + jmsg.getJMSTimestamp());
			}
		} catch (JMSException e) {
			log.error("Something went wrong printing messages info.", e);
		}
	}

	/** Removes all messages from Queue by deleting all messages. */
	public static boolean emptyQueue(final MBeanServerConnection mserver, final String queueName) {

		ObjectName eq;
		boolean ok = true;
		try {
			eq = new ObjectName("jboss.messaging.destination:service=Queue,name=" + queueName);
			mserver.invoke(eq, "removeAllMessages", null, null);
		} catch (Exception e) {
			ok = false;
		}
		if (ok)
			ok = (getQueueSize(mserver, queueName) == 0);
		return ok;
	}

	/** Removes all messages from queue by reading (consuming) them all. */
	public static boolean drainQueue(final InitialContext ic, final String queueName, int amount) {

		int msgsOnQueue = getQueueSize(getMBeanServer(ic), queueName);
		int msgsDrained = 0;
		if (msgsOnQueue <= 0) {
			log.info("Queue " + queueName + " is already empty");
		} else {
			Connection con = null;
			try {
				Queue queue = (Queue)ic.lookup("queue/" + queueName);
				ConnectionFactory cf = getConnectionFactory(ic);
				con = cf.createConnection();
				Session session = con.createSession(false, Session.AUTO_ACKNOWLEDGE);
				MessageConsumer consumer = session.createConsumer(queue);
				con.start();
				boolean receivedMsg = true; 
				while (msgsDrained < amount && msgsDrained < msgsOnQueue && receivedMsg) {
					receivedMsg = (consumer.receive(1500) != null);
					msgsDrained++;
					if (msgsDrained % 10 == 0)
						System.out.println("Drained " + msgsDrained + " messages.");
				}
				log.info("Removed " + msgsDrained + " messages from queue " + queueName +
						",\ncurrent number of messages on queue: " + getQueueSize(getMBeanServer(ic), queueName));
			} catch (Exception e) {
				log.error("Clearing queue failed.", e);
			} finally {
				closeConnection(con);
			}
		}
		return (msgsDrained == msgsOnQueue);
	}

	public static boolean sendMessage(final InitialContext ic, final String queueName, final String text) {

		Connection con = null;
		boolean error = false;
		try {
			Queue queue = (Queue)ic.lookup("queue/" + queueName);
			ConnectionFactory cf = getConnectionFactory(ic);
			con = cf.createConnection();
			Session session = con.createSession(false, Session.AUTO_ACKNOWLEDGE);

			MessageProducer sender = session.createProducer(queue);
			TextMessage message = session.createTextMessage(text);
			sender.send(message);

			log.debug("Message send to queue " + queueName + ": " + text);
		} catch (Exception e) {
			log.error("Could not send message on " + queueName + ": ", e);
			error = true;
		} finally {
			closeConnection(con);
		}
		return !error;
	}

	public static Message receiveMessage(final InitialContext ic, final String queueName) {

		Connection con = null;
		Message msg = null;
		try {
			Queue queue = (Queue)ic.lookup("queue/" + queueName);
			ConnectionFactory cf = getConnectionFactory(ic);
			con = cf.createConnection();
			Session session = con.createSession(false, Session.AUTO_ACKNOWLEDGE);
			MessageConsumer consumer =  session.createConsumer(queue);
			con.start();
			msg = (Message)consumer.receive(2000);

			log.debug("Message received from queue " + queueName + ".");
		} catch (Exception e) {
			log.error("Could not receive message on " + queueName + ": ", e);
			msg = null;
		} finally {
			closeConnection(con);
		}
		return msg;
	}
	
	public static String getMessageContentsAsString(final Message msg) {
		
		String result = "";
		
		if (msg instanceof TextMessage) {
			try {
				result = ( ((TextMessage)msg).getText() );
			} catch (JMSException jmse) {
				result = (jmse.getMessage());
			}
			
		
		} else if (msg instanceof ObjectMessage) {
			Serializable msgo = null;
			try {
				msgo = ((ObjectMessage)msg).getObject();
			} catch (ClassCastException cce) {
				if (msgo == null) {
					result = "Object message contains no object.";
				} else {
					result = "Object Message does not contain a known object." +
						"\nObject name : " + msgo.getClass().getName() + 
						"\nObject description: " + msgo.toString();
				}
			} catch (JMSException jmse) {
				result = jmse.getMessage();
			}
			
			if (msgo != null) {
				ByteArrayOutputStream xmlout = new ByteArrayOutputStream();
				XMLEncoder enc = new XMLEncoder(xmlout);
				enc.writeObject(msgo);
				enc.close();
				try {
					result = xmlout.toString("UTF-8");
				} catch (UnsupportedEncodingException e1) {
					result = "Could not use UTF-8 encoding for use with XML.\n" + e1.getMessage();
				}
				try {xmlout.close(); } catch (Exception e) {}
			}
		}
		return result;
	}

	/** Creates a queue with the given name if it does not already exist. */
	public static boolean deployQueue(final InitialContext ic, final String queueName) {

		if (queueExists(ic, queueName))
			return true;
		boolean queueCreated = false;
		try {
			MBeanServerConnection mBeanServer = (MBeanServerConnection)ic.lookup("jmx/invoker/RMIAdaptor");
			ObjectName serverObjectName = new ObjectName("jboss.messaging:service=ServerPeer");
			mBeanServer.invoke(serverObjectName, "deployQueue",
					new Object[] {queueName, "queue/" + queueName},
					new String[] {"java.lang.String", "java.lang.String"});
			queueCreated = true;
		} catch (Exception e) {}
		return queueCreated;
	}

	/** Destroys a queue if it exists. All queue information is deleted, also from database.*/
	public static boolean destroyQueue(final InitialContext ic, final String queueName) {

		if (!queueExists(ic, queueName))
			return true;
		boolean queueRemoved = false;
		try {
			MBeanServerConnection mBeanServer = (MBeanServerConnection)ic.lookup("jmx/invoker/RMIAdaptor");
			ObjectName serverObjectName = new ObjectName("jboss.messaging:service=ServerPeer");
			mBeanServer.invoke(serverObjectName, "destroyQueue",
					new Object[] {"queue/" + queueName},
					new String[] {"java.lang.String"});
			queueRemoved = true;
			if (log != null) {
				log.info("Destroyed queue " + queueName);
			}
		} catch (Exception e) {
			if (log != null)
				log.error("Failed to destroy queue " + queueName, e);
		}
		return queueRemoved;
	}
	
	public static ArrayList<ObjectName> getBeanNames(final MBeanServerConnection msc, final String nameFilter) 
			throws MalformedObjectNameException, IOException {
		
		ObjectName om = new ObjectName(nameFilter);
		Set<ObjectName> beanNames = (Set<ObjectName>)msc.queryNames(om, null);
		ArrayList<ObjectNameSortable> bns = new ArrayList<ObjectNameSortable>();
		for (ObjectName on : beanNames) {
			bns.add(thisInstance.new ObjectNameSortable(on));
		}
		Collections.sort(bns);
		ArrayList<ObjectName> onList = new ArrayList<ObjectName>();
		for (ObjectNameSortable ons : bns) onList.add(ons.getObjectName());
		bns.clear();
		return onList;
	}

	public static void printBeanNames(final MBeanServerConnection msc) throws MalformedObjectNameException, NullPointerException {
		printBeanNames(msc, "*:*");
	}
	
	public static void printBeanNames(final MBeanServerConnection msc, final String nameFilter) 
			throws MalformedObjectNameException {

		try {
			ArrayList<ObjectName> bns = getBeanNames(msc, nameFilter);
			for (ObjectName on : bns) System.out.println(on.toString());
			bns.clear();
			bns = null;
		} catch (IOException e) {
			log.error("Error printing list of beans.");
		}
	}

	class ObjectNameSortable implements Comparable<ObjectNameSortable> {

		private final ObjectName on;
		public ObjectNameSortable(final ObjectName on) {
			super();
			this.on = on;
		}
		public ObjectName getObjectName() {
			return on;
		}
		public int compareTo(final ObjectNameSortable o) {
			return toString().compareTo(o.toString());
		}
		public String toString() {
			return on.getDomain() + on.toString();
		}
	}
	
	/** Shows JMX-queue properties. */
	public static void main(final String[] args) {
		
		try {
			SecurityClient sc = SecurityClientFactory.getSecurityClient();
			sc.setVmwideAssociation(true);
			
			sc.setSimple("admin", "admin");
			
			sc.login();
			Properties p = new Properties();
			p.put(InitialContext.INITIAL_CONTEXT_FACTORY, "org.jnp.interfaces.NamingContextFactory");
			p.put(InitialContext.URL_PKG_PREFIXES, "org.jboss.naming:org.jnp.interfaces");
			p.put(InitialContext.PROVIDER_URL, "jnp://localhost:1099");

			MBeanServerConnection mbsc = getMBeanServer(new InitialContext(p)); 
			
			ObjectName probQ = new ObjectName("jboss.messaging.destination:service=Queue,name=DLQ");
			
			MBeanInfo beanInfo = mbsc.getMBeanInfo(probQ);
			MBeanAttributeInfo[] attInfo = beanInfo.getAttributes();
			for(int i = 0; i < attInfo.length; i++)
				System.out.println(attInfo[i].getName() + " (" + attInfo[i].getType() + "): " + attInfo[i].getDescription());
			sc.logout();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * DOES NOT WORK - Cannot get this to work, keeps throwing RMI connector error.
	 */
	@SuppressWarnings("unused")
	private static void connectXML() {
		/**
		 * The URL to the JBoss RMI connector.
		 * The port is set in jboss_service.xml via
		 * <attribute name="RmiPort">1098</attribute>
		 * 
		 * Following runtime variables are also needed:
		 * -Dcom.sun.management.jmxremote.port=9900
	                 -Dcom.sun.management.jmxremote.ssl=false
	                 -Dcom.sun.management.jmxremote.authenticate=false

		 * 
		 */
		String jmxServerURL = "service:jmx:rmi:///jndi/rmi://localhost:9900/jmxrmi";
		//String jmxServerURL = "service:jmx:rmi:///jndi/rmi://localhost:1090/jmxconnector";

		JMXServiceURL url = null;
		JMXConnector jmxc = null;
		MBeanServerConnection mbsc = null;
		try {
			System.out.println("Connecting to: " + InetAddress.getLocalHost().getHostName());
			url = new JMXServiceURL(jmxServerURL);
			System.out.println("Fetching JMXConnector");
			jmxc = JMXConnectorFactory.connect(url, null);
			System.out.println("Fetching bean server connection");
			mbsc = jmxc.getMBeanServerConnection();
			System.out.println("Connected to: " + url.getURLPath());
		} catch (Exception e) {
			log.error("Cannot open connection: ", e);
		} finally {
			if (jmxc != null) {
				try { jmxc.close();} catch (IOException unused) {}
			}
		}
	}
	/* Code unused, kept here for reference.
	System.out.println(ctx.getEnvironment().toString());
	NamingEnumeration<NameClassPair> names = ctx.list("");
	while (names.hasMore()) {
		NameClassPair nc = names.next();
		System.out.println(nc.getName() + " " + nc.getClassName());
	}
	 */


}
