package nl.intercommit.mario.tools.jbossmon;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Timer;
import java.util.TimerTask;

import javax.management.MBeanServerConnection;
import javax.swing.table.AbstractTableModel;

import net.java.dev.fwutil.Log;
import nl.intercommit.mario.tools.JmsUtil;
import nl.intercommit.mario.tools.MarioContext;

import org.jboss.jms.server.messagecounter.MessageStatistics;

public class ServerQueueMon extends AbstractTableModel {

	public List<String> qNames;
	public HashMap<String, QueueProps> qInfo = new HashMap<String, QueueProps>();
	public static int ReconnectDelaySeconds = 20;
	private long serverStatUpdateDelay = 10000;
	private static final long serialVersionUID = 3926822294444254175L;
	private final Timer secondsTimer;
	private MarioContext mc;
	private MBeanServerConnection mserver;
	private boolean connectionOK;
	private int reconnectDelay;
	private final JbServerMon serverWindow;
	private boolean stopMonitor;
	private boolean disableMsgStatsOnExit = false;

	public ServerQueueMon(final MarioContext mc, final JbServerMon smon) {
		super();
		this.mc = mc;
		serverWindow = smon;
		stopMonitor = false;
		mserver = JmsUtil.getMBeanServer(mc.ic);
		connectionOK = (mserver != null);
		reconnectDelay = ReconnectDelaySeconds;
		getQueueNames();
		// Do NOT use a common timer (e.g. Util.getTimer()), this will hang when a new task 
		// is scheduled by a second instance of this class.
		secondsTimer = new Timer();
		// Retrieve all the statistics and schedule a task to do so again in the near future.
		updateData(true);
		//try { Thread.sleep(500);} catch (InterruptedException e) {}
		// Retrieve only the queue-depth and schedule a task to do so again in the near future.
		updateData(false);
	}

	public int getColumnCount() {
		return 8;
	}

	public int getRowCount() {
		return qNames.size();
	}

	public String getColumnName(final int column) {

		String columnName;
		switch (column) {
		case 0 : columnName = "Queue name"; break;
		case 1 : columnName = "On queue messages"; break;
		case 2 : columnName = "Peak messages on queue"; break;
		case 3 : columnName = "Consumers"; break;
		case 4 : columnName = "Average per second"; break;
		case 5 : columnName = "Total messages"; break;
		case 6 : columnName = "Average per second over " + QueueProps.avgPeriod + " periods"; break;
		case 7 : columnName = "Peak average per second"; break;
		default: columnName = "Unknown";
		};
		return columnName;
	}

	public Object getValueAt(final int row, final int col) {

		if (col == 0)
			return qNames.get(row);
		String qn = qNames.get(row);
		QueueProps qp = qInfo.get(qn);
		switch (col) {
		case 1: return qp.queueDepth;
		case 2: return qp.queueDepthPeak;
		case 3: return qp.msgConsumers;
		case 4 : return qp.avgPerSecLast;
		case 5 : return qp.msgTotal;
		case 6 : return qp.avgPerSecPeriod;
		case 7 : return qp.avgPerSecPeak;
		}
		return "Unknown";
	}

	public Class<?> getColumnClass(final int col) {

		switch (col) {
		case 0 : return String.class;
		case 1 : return Integer.class;
		case 2 : return Integer.class;
		case 3 : return Integer.class;
		case 4 : return Integer.class;
		case 5 : return Integer.class;
		case 6 : return Integer.class;
		case 7 : return Integer.class;
		}
		return super.getColumnClass(col);
	}

	public void getQueueNames() {

		qNames = JmsUtil.getQueueNames(mc.ic);
		if (qNames == null) {
			qNames = new ArrayList<String>();
			serverWindow.setStatus("Retrieved no queues from server.");
		}
		Collections.sort(qNames);
		if (qNames.size() > 0) {
			serverStatUpdateDelay = JmsUtil.getQueueStatsUpdateDelay(mserver);
			serverWindow.setStatus("Retrieved " + qNames.size() + " queues from server." +
					" Update delay set to " + serverStatUpdateDelay + " milliseconds.");
		}
		fireTableDataChanged();
	}
	
	public MessageStatistics getQueueStat(final List<MessageStatistics> qStat, final String queueName) {

		if (qStat == null)
			return null;
		int statIndex = -1;
		int count = 0;
		while (count < qStat.size() && statIndex == -1) {
			if (qStat.get(count).getName().endsWith("." + queueName))
				statIndex = count;
			count++;
		}
		if (statIndex == -1) {
			Log.logError("Could not find statistics for queue " + queueName);
			return null;
		}
		return qStat.get(statIndex);
	}

	public void stopMonitor() {
		
		stopMonitor = true;
		serverWindow.setStatus("Monitoring stopped");
	}

	public void startMonitor() {
		
		stopMonitor = false;
		serverWindow.setStatus("Monitoring started");
		updateData(true);
	}

	public void refreshQueues() {
		getQueueNames();
	}

	public void clearQueue(final String queueName) {
		if (JmsUtil.emptyQueue(mserver, queueName)) {
			serverWindow.setStatus("All messages removed from queue " + queueName);
		} else {
			serverWindow.setStatus("Could not remove messages from queue " + queueName + "got an error.");
		}
	}

	public void updateData(final boolean updateStats) {

		if (stopMonitor) {
			return;
		}
		//Log.log("Updating data, connection is " + connectionOK + ", stat update is " + updateStats);
		if (!connectionOK && !updateStats) {
			if (reconnectDelay > 1) {
				reconnectDelay--;
				serverWindow.setStatus("Retrying connection in " + reconnectDelay + " seconds");
			} else {
				reConnect();
				if (connectionOK) {
					if (updateStats) {
						secondsTimer.schedule(new qInfoUpdateTask(), 1000);
					} else {
						secondsTimer.schedule(new qStatsUpdateTask(), 1000);
					}
				}
			}
		}
		if (!connectionOK) {
			if (!updateStats)
				secondsTimer.schedule(new qInfoUpdateTask(), 1000);
			return;
		}
		HashMap<String, Object> qVal;
		QueueProps qp;
		String qn;
		int qcount = 0;
		String qstatsError = null;
		List<MessageStatistics> qStats = null;
		if (updateStats) {
			qStats = JmsUtil.getQueueCounterStats(mserver);
			if (qStats == null) {
				Log.logError(serverWindow.getServerURL() + " - Unable to retrieve queue statistics.");
			}
		}
		MessageStatistics qStat = null;
		while (connectionOK && qcount < qNames.size()) {
			qn = qNames.get(qcount);
			qp = qInfo.get(qn);
			if (qp == null) {
				qp = new QueueProps();
				qInfo.put(qn, qp);
			}
			//Log.log("Retrieving queue data from server " + qn);
			qVal = JmsUtil.getQueueAttValues(mserver, qn, QueueProps.queueAtts);
			if (qVal == null) {
				Log.logError("Failed to get queue attribute values for queue " + qn);
				connectionOK = false;
			}
			if (qVal != null) {
				Integer i = (Integer)qVal.get(QueueProps.NumberOfMsgs);
				qp.queueDepth = i.intValue();
				if (qp.queueDepthPeak < qp.queueDepth)
					qp.queueDepthPeak = qp.queueDepth;
				i = (Integer)qVal.get(QueueProps.NumberOfConsumers);
				qp.msgConsumers = i.intValue();
			}
			if (updateStats && qStats != null) {
				qStat = getQueueStat(qStats, qn);
				if (qStat != null) {
					String s = qp.addStats(qStat);
					if (qstatsError == null && s != null)
						qstatsError = s;
				}
			}
			qcount++;
		}
		if (qstatsError != null)
			Log.logWarning(qstatsError);
		fireTableRowsUpdated(0, getRowCount()-1);
		if (!connectionOK) {
			serverWindow.setStatus("Communication error, refreshing queue names.");
			getQueueNames();
			if (qNames.size() > 0) {
				connectionOK = true;
			} else {
				disConnect();
			}
		}
		if (updateStats) {
			secondsTimer.schedule(new qStatsUpdateTask(), serverStatUpdateDelay);
		} else {
			secondsTimer.schedule(new qInfoUpdateTask(), 1000);
		}
	}

	private void disConnect() {

		serverWindow.setStatus("Connection lost, taking time-out.");
		mc.close();
		if (mc.icEnv == null) {
			Log.logError("Cannot reconnect, server parameters are unknown.");
			stopMonitor();
			serverWindow.dispose();
			return;
		}
		reconnectDelay = ReconnectDelaySeconds;
	}

	public void reConnect() {

		boolean error = false;
		serverWindow.setStatus("Retrying connection ...");
		try {
			mc.reconnect();
			if (mc.ic == null) throw new Exception("Failed to reconnect to JBoss server.");
			mserver = JmsUtil.getMBeanServer(mc.ic);
			serverStatUpdateDelay = JmsUtil.getQueueStatsUpdateDelay(mserver);
			getQueueNames();
			if (qNames.size() == 0) {
				error = true;
			} else {
				JmsUtil.enableQueueStats(mserver, true);
			}
		} catch (Exception e) {
			Log.logError(e);
			error = true;
		}
		connectionOK = !error;
		if (connectionOK) {
			serverWindow.setStatus("Connection OK");
		} else {
			disConnect();
		}
	}

	class qInfoUpdateTask extends TimerTask {
		public void run() {
			updateData(false);
		}
	}

	class qStatsUpdateTask extends TimerTask {
		public void run() {
			updateData(true);
		}
	}

	public MarioContext getMC() {
		return mc;
	}

	public void resetStats() {

		QueueProps qp;
		for (int i = 0; i < qNames.size(); i++) {
			qp = qInfo.get(qNames.get(i));
			if (qp != null)
				qp.clearStats();
		}
	}

	public void resetCounters() {

		if (JmsUtil.resetQueueStats(mserver)) {
			resetStats();
			serverWindow.setStatus("Counters reset on JBoss server.");
		} else {
			serverWindow.setStatus("Failed to reset counters on JBoss server. See log for more info.");
		}
	}

	public void disableMsgStats(final boolean value) {
		disableMsgStatsOnExit = value;
	}


	public void dispose() {

		// Java 5: Just cancelling the task as in:
		secondsTimer.cancel(); // Does work in Java 6.
		// will not work properly, it leaves the ServerMon window in an active state
		// without the option to show it. Must use stopMonitor in order to properly close the frame.
		stopMonitor(); // Shows message in log "Monitoring stopped".
		qNames.clear();
		qInfo.clear();
		if (disableMsgStatsOnExit) {
			if (JmsUtil.enableQueueStats(mserver, false)) {
				Log.log("Message statistics disabled on server.");
			} else {
				Log.log("Failed to disable message statistics on server.");
			}
		}
		mc.close();
	}
}
